"""Base class for platform implementations
"""
import ctypes
from OpenGL.platform import ctypesloader
from OpenGL._bytes import as_8_bit
import sys
from OpenGL import _configflags
from OpenGL import logs, MODULE_ANNOTATIONS

class lazy_property( object ):
    def __init__( self, function ):
        self.fget = function 
    def __get__( self, obj, cls ):
        value = self.fget( obj )
        setattr( obj, self.fget.__name__, value)
        return value 

class _CheckContext( object ):
    def __init__( self, func, ccisvalid ):
        self.func = func 
        self.ccisvalid = ccisvalid
    def __setattr__( self, key, value ):
        if key not in ('func','ccisvalid'):
            return setattr( self.func, key, value )
        else:
            self.__dict__[key] = value 
    def __repr__( self ):
        if getattr( self.func, '__doc__', None ):
            return self.func.__doc__    
        else:
            return repr( self.func )
    def __getattr__( self, key ):
        if key != 'func':
            return getattr(self.func, key )
        raise AttributeError( key )
    def __call__( self, *args, **named ):
        if not self.ccisvalid():
            from OpenGL import error
            raise error.NoContext( self.func.__name__, args, named )
        return self.func( *args, **named )

def _find_module( exclude = (__name__,)):
    frame = sys._getframe()
    while frame and '__name__' in frame.f_globals:
        if exclude:
            if not frame.f_globals['__name__'] in exclude:
                return frame.f_globals['__name__']
            
        else:
            return frame.f_globals['__name__']
        frame = frame.f_back
    return None

class BasePlatform( object ):
    """Base class for per-platform implementations
    
    Attributes of note:
    
        EXPORTED_NAMES -- set of names exported via the platform 
            module's namespace...
    
        GL, GLU, GLUT, GLE, GLES1, GLES2, GLES3 -- ctypes libraries
    
        DEFAULT_FUNCTION_TYPE -- used as the default function 
            type for functions unless overridden on a per-DLL
            basis with a "FunctionType" member
        
        GLUT_GUARD_CALLBACKS -- if True, the GLUT wrappers 
            will provide guarding wrappers to prevent GLUT 
            errors with uninitialised GLUT.
        
        EXTENSIONS_USE_BASE_FUNCTIONS -- if True, uses regular
            dll attribute-based lookup to retrieve extension 
            function pointers.
    """
    
    EXPORTED_NAMES = [
        'GetCurrentContext','CurrentContextIsValid',
        'createBaseFunction', 'createExtensionFunction', 'copyBaseFunction',
        'getGLUTFontPointer','nullFunction',
        'GLUT_GUARD_CALLBACKS',
    ]

    
    DEFAULT_FUNCTION_TYPE = None
    GLUT_GUARD_CALLBACKS = False
    EXTENSIONS_USE_BASE_FUNCTIONS = False
    
    def install( self, namespace ):
        """Install this platform instance into the platform module"""
        for name in self.EXPORTED_NAMES:
            namespace[ name ] = getattr(self,name,None)
        namespace['PLATFORM'] = self
        return self
    
    def functionTypeFor( self, dll ):
        """Given a DLL, determine appropriate function type..."""
        if hasattr( dll, 'FunctionType' ):
            return dll.FunctionType
        else:
            return self.DEFAULT_FUNCTION_TYPE
    
    def errorChecking( self, func, dll, error_checker=None ):
        """Add error checking to the function if appropriate"""
        from OpenGL import error
        if error_checker and _configflags.ERROR_CHECKING:
            #GLUT spec says error-checking is basically undefined...
            # there *may* be GL errors on GLUT calls that e.g. render 
            # geometry, but that's all basically "maybe" stuff...
            func.errcheck = error_checker.glCheckError
        return func
    def wrapContextCheck( self, func, dll ):
        """Wrap function with context-checking if appropriate"""
        if _configflags.CONTEXT_CHECKING and dll is self.GL and func.__name__ not in (
            'glGetString',
            'glGetStringi',
            'glGetIntegerv',
        ) and not func.__name__.startswith( 'glX' ):
            return _CheckContext( func, self.CurrentContextIsValid )
        return func 
    def wrapLogging( self, func ):
        """Wrap function with logging operations if appropriate"""
        return logs.logOnFail( func, logs.getLog( 'OpenGL.errors' ))
    
    def finalArgType( self, typ ):
        """Retrieve a final type for arg-type"""
        if typ == ctypes.POINTER( None ) and not getattr( typ, 'final',False):
            from OpenGL.arrays import ArrayDatatype
            return ArrayDatatype
        else:
            return typ
    def constructFunction( 
        self,
        functionName, dll, 
        resultType=ctypes.c_int, argTypes=(),
        doc = None, argNames = (),
        extension = None,
        deprecated = False,
        module = None,
        force_extension = False,
        error_checker = None,
    ):
        """Core operation to create a new base ctypes function
        
        raises AttributeError if can't find the procedure...
        """
        is_core = (not extension) or extension.split('_')[1] == 'VERSION'
        if (not is_core) and not self.checkExtension( extension ):
            raise AttributeError( """Extension not available""" )
        argTypes = [ self.finalArgType( t ) for t in argTypes ]
            
        if force_extension or ((not is_core) and (not self.EXTENSIONS_USE_BASE_FUNCTIONS)):
            # what about the VERSION values???
            pointer = self.getExtensionProcedure( as_8_bit(functionName) )
            if pointer:
                func = self.functionTypeFor( dll )(
                    resultType,
                    *argTypes
                )(
                    pointer
                )
            else:
                raise AttributeError( """Extension %r available, but no pointer for function %r"""%(extension,functionName))
        else:
            func = ctypesloader.buildFunction(
                self.functionTypeFor( dll )(
                    resultType,
                    *argTypes
                ),
                functionName,
                dll,
            )
        func.__doc__ = doc 
        func.argNames = list(argNames or ())
        func.__name__ = functionName
        func.DLL = dll
        func.extension = extension
        func.deprecated = deprecated
        func = self.wrapLogging( 
            self.wrapContextCheck(
                self.errorChecking( func, dll, error_checker=error_checker ),
                dll,
            )
        )
        if MODULE_ANNOTATIONS:
            if not module:
                module = _find_module( )
            if module:
                func.__module__ = module
        return func

    def createBaseFunction( 
        self,
        functionName, dll, 
        resultType=ctypes.c_int, argTypes=(),
        doc = None, argNames = (),
        extension = None,
        deprecated = False,
        module = None,
        error_checker = None,
    ):
        """Create a base function for given name
        
        Normally you can just use the dll.name hook to get the object,
        but we want to be able to create different bindings for the 
        same function, so we do the work manually here to produce a
        base function from a DLL.
        """
        from OpenGL import wrapper
        result = None
        try:
            if (
                _configflags.FORWARD_COMPATIBLE_ONLY and 
                dll is self.GL and 
                deprecated
            ):
                result = self.nullFunction(
                    functionName, dll=dll,
                    resultType=resultType, 
                    argTypes=argTypes,
                    doc = doc, argNames = argNames,
                    extension = extension,
                    deprecated = deprecated,
                    error_checker = error_checker,
                )
            else:
                result = self.constructFunction(
                    functionName, dll, 
                    resultType=resultType, argTypes=argTypes,
                    doc = doc, argNames = argNames,
                    extension = extension,
                    error_checker = error_checker,
                )
        except AttributeError as err:
            result = self.nullFunction( 
                functionName, dll=dll,
                resultType=resultType, 
                argTypes=argTypes,
                doc = doc, argNames = argNames,
                extension = extension,
                error_checker = error_checker,
            )
        if MODULE_ANNOTATIONS:
            if not module:
                module = _find_module( )
            if module:
                result.__module__ = module
        return result
    def checkExtension( self, name ):
        """Check whether the given extension is supported by current context"""
#        if not name or name in ('GL_VERSION_GL_1_0', 'GL_VERSION_GL_1_1'):
#            return True
#        if name.startswith( 'EGL_' ) or name.startswith( 'GLX_' ) or name.startswith( 'WGL_' ):
#            # we can't check these extensions, have to rely on the function resolution
#            return True
        if not name:
            return True
        context = self.GetCurrentContext()
        if context:
            from OpenGL import contextdata
            set = contextdata.getValue( 'extensions', context=context )
            if set is None:
                set = {}
                contextdata.setValue( 
                    'extensions', set, context=context, weak=False 
                )
            current = set.get( name )
            if current is None:
                from OpenGL import extensions
                result = extensions.ExtensionQuerier.hasExtension( name )
                set[name] = result 
                return result
            return current
        else:
            from OpenGL import extensions
            return extensions.ExtensionQuerier.hasExtension( name )
    createExtensionFunction = createBaseFunction

    def copyBaseFunction( self, original ):
        """Create a new base function based on an already-created function
        
        This is normally used to provide type-specific convenience versions of
        a definition created by the automated generator.
        """
        from OpenGL import wrapper, error
        if isinstance( original, _NullFunctionPointer ):
            return self.nullFunction(
                original.__name__,
                original.DLL,
                resultType = original.restype,
                argTypes= original.argtypes,
                doc = original.__doc__,
                argNames = original.argNames,
                extension = original.extension,
                deprecated = original.deprecated,
                error_checker = original.error_checker,
            )
        elif hasattr( original, 'originalFunction' ):
            original = original.originalFunction
        return self.createBaseFunction(
            original.__name__, original.DLL, 
            resultType=original.restype, argTypes=original.argtypes,
            doc = original.__doc__, argNames = original.argNames,
            extension = original.extension,
            deprecated = original.deprecated,
            error_checker = original.errcheck,
        )
    def nullFunction( 
        self,
        functionName, dll,
        resultType=ctypes.c_int, 
        argTypes=(),
        doc = None, argNames = (),
        extension = None,
        deprecated = False,
        module = None,
        error_checker = None,
    ):
        """Construct a "null" function pointer"""
        if deprecated:
            base = _DeprecatedFunctionPointer
        else:
            base = _NullFunctionPointer
        cls = type( functionName, (base,), {
            '__doc__': doc,
            'deprecated': deprecated,
        } )
        if MODULE_ANNOTATIONS:
            if not module:
                module = _find_module( )
            if module:
                cls.__module__ = module
        return cls(
            functionName, dll, resultType, argTypes, argNames, extension=extension, doc=doc,
            error_checker = error_checker,
        )
    def GetCurrentContext( self ):
        """Retrieve opaque pointer for the current context"""
        raise NotImplementedError( 
            """Platform does not define a GetCurrentContext function""" 
        )
    def getGLUTFontPointer(self, constant ):
        """Retrieve a GLUT font pointer for this platform"""
        raise NotImplementedError( 
            """Platform does not define a GLUT font retrieval function""" 
        )
    # names that are normally just references to other items...
    @lazy_property
    def CurrentContextIsValid( self ):
        return self.GetCurrentContext
    @lazy_property
    def OpenGL(self): return self.GL

class _NullFunctionPointer( object ):
    """Function-pointer-like object for undefined functions"""
    def __init__( 
        self, name, dll, resultType, argTypes, argNames, 
        extension=None, doc=None, deprecated=False,
        error_checker = None,
    ):
        from OpenGL import error
        self.__name__ = name
        self.DLL = dll
        self.argNames = argNames
        self.argtypes = argTypes
        self.errcheck = None
        self.restype = resultType
        self.extension = extension
        self.doc = doc
        self.deprecated = deprecated
        self.error_checker = error_checker
    resolved = False
    def __nonzero__( self ):
        """Make this object appear to be NULL"""
        if self.extension and not self.resolved:
            self.load()
        return self.resolved
    __bool__ = __nonzero__
    def load( self ):
        """Attempt to load the function again, presumably with a context this time"""
        from OpenGL import platform
        try:
            func = platform.PLATFORM.constructFunction(
                self.__name__, self.DLL, 
                resultType=self.restype, 
                argTypes=self.argtypes,
                doc = self.doc, 
                argNames = self.argNames,
                extension = self.extension,
                error_checker = self.error_checker,
            )
        except AttributeError as err:
            return None 
        else:
            # now short-circuit so that we don't need to check again...
            self.__class__.__call__ = staticmethod( func.__call__ )
            self.resolved = True
            return func
        return None
    def __call__( self, *args, **named ):
        if self.load():
            return self( *args, **named )
        else:
            from OpenGL import error
            raise error.NullFunctionError(
                """Attempt to call an undefined function %s, check for bool(%s) before calling"""%(
                    self.__name__, self.__name__,
                )
            )

class _DeprecatedFunctionPointer( _NullFunctionPointer ):
    deprecated = True
    def __call__( self, *args, **named ):
        from OpenGL import error
        raise error.NullFunctionError(
            """Attempt to call a deprecated function %s while OpenGL in FORWARD_COMPATIBLE_ONLY mode.  Set OpenGL.FORWARD_COMPATIBLE_ONLY to False to use legacy entry points"""%(
                self.__name__,
            )
        )



'''OpenGL extension NV.shader_atomic_counters

This module customises the behaviour of the 
OpenGL.raw.GL.NV.shader_atomic_counters to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds upon the ARB_shader_atomic_counters and
	NV_gpu_program5 extensions to provide assembly language support for
	incrementing, decrementing, and querying the values of atomic counters
	stored in buffer object memory.
	
	The extension uses the same set of atomic counter buffer binding points as
	the ARB_shader_atomic_counters extension; applications using this
	extension should use the APIs specified there to bind buffers.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/shader_atomic_counters.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.shader_atomic_counters import *
from OpenGL.raw.GL.NV.shader_atomic_counters import _EXTENSION_NAME

def glInitShaderAtomicCountersNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_VERSION_GL_3_2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_VERSION_GL_3_2',error_checker=_errors._error_checker)
GL_ALREADY_SIGNALED=_C('GL_ALREADY_SIGNALED',0x911A)
GL_CONDITION_SATISFIED=_C('GL_CONDITION_SATISFIED',0x911C)
GL_CONTEXT_COMPATIBILITY_PROFILE_BIT=_C('GL_CONTEXT_COMPATIBILITY_PROFILE_BIT',0x00000002)
GL_CONTEXT_CORE_PROFILE_BIT=_C('GL_CONTEXT_CORE_PROFILE_BIT',0x00000001)
GL_CONTEXT_PROFILE_MASK=_C('GL_CONTEXT_PROFILE_MASK',0x9126)
GL_DEPTH_CLAMP=_C('GL_DEPTH_CLAMP',0x864F)
GL_FIRST_VERTEX_CONVENTION=_C('GL_FIRST_VERTEX_CONVENTION',0x8E4D)
GL_FRAMEBUFFER_ATTACHMENT_LAYERED=_C('GL_FRAMEBUFFER_ATTACHMENT_LAYERED',0x8DA7)
GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS=_C('GL_FRAMEBUFFER_INCOMPLETE_LAYER_TARGETS',0x8DA8)
GL_GEOMETRY_INPUT_TYPE=_C('GL_GEOMETRY_INPUT_TYPE',0x8917)
GL_GEOMETRY_OUTPUT_TYPE=_C('GL_GEOMETRY_OUTPUT_TYPE',0x8918)
GL_GEOMETRY_SHADER=_C('GL_GEOMETRY_SHADER',0x8DD9)
GL_GEOMETRY_VERTICES_OUT=_C('GL_GEOMETRY_VERTICES_OUT',0x8916)
GL_INT_SAMPLER_2D_MULTISAMPLE=_C('GL_INT_SAMPLER_2D_MULTISAMPLE',0x9109)
GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910C)
GL_LAST_VERTEX_CONVENTION=_C('GL_LAST_VERTEX_CONVENTION',0x8E4E)
GL_LINES_ADJACENCY=_C('GL_LINES_ADJACENCY',0x000A)
GL_LINE_STRIP_ADJACENCY=_C('GL_LINE_STRIP_ADJACENCY',0x000B)
GL_MAX_COLOR_TEXTURE_SAMPLES=_C('GL_MAX_COLOR_TEXTURE_SAMPLES',0x910E)
GL_MAX_DEPTH_TEXTURE_SAMPLES=_C('GL_MAX_DEPTH_TEXTURE_SAMPLES',0x910F)
GL_MAX_FRAGMENT_INPUT_COMPONENTS=_C('GL_MAX_FRAGMENT_INPUT_COMPONENTS',0x9125)
GL_MAX_GEOMETRY_INPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_INPUT_COMPONENTS',0x9123)
GL_MAX_GEOMETRY_OUTPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_OUTPUT_COMPONENTS',0x9124)
GL_MAX_GEOMETRY_OUTPUT_VERTICES=_C('GL_MAX_GEOMETRY_OUTPUT_VERTICES',0x8DE0)
GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS=_C('GL_MAX_GEOMETRY_TEXTURE_IMAGE_UNITS',0x8C29)
GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS=_C('GL_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS',0x8DE1)
GL_MAX_GEOMETRY_UNIFORM_COMPONENTS=_C('GL_MAX_GEOMETRY_UNIFORM_COMPONENTS',0x8DDF)
GL_MAX_INTEGER_SAMPLES=_C('GL_MAX_INTEGER_SAMPLES',0x9110)
GL_MAX_SAMPLE_MASK_WORDS=_C('GL_MAX_SAMPLE_MASK_WORDS',0x8E59)
GL_MAX_SERVER_WAIT_TIMEOUT=_C('GL_MAX_SERVER_WAIT_TIMEOUT',0x9111)
GL_MAX_VERTEX_OUTPUT_COMPONENTS=_C('GL_MAX_VERTEX_OUTPUT_COMPONENTS',0x9122)
GL_OBJECT_TYPE=_C('GL_OBJECT_TYPE',0x9112)
GL_PROGRAM_POINT_SIZE=_C('GL_PROGRAM_POINT_SIZE',0x8642)
GL_PROVOKING_VERTEX=_C('GL_PROVOKING_VERTEX',0x8E4F)
GL_PROXY_TEXTURE_2D_MULTISAMPLE=_C('GL_PROXY_TEXTURE_2D_MULTISAMPLE',0x9101)
GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY=_C('GL_PROXY_TEXTURE_2D_MULTISAMPLE_ARRAY',0x9103)
GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION=_C('GL_QUADS_FOLLOW_PROVOKING_VERTEX_CONVENTION',0x8E4C)
GL_SAMPLER_2D_MULTISAMPLE=_C('GL_SAMPLER_2D_MULTISAMPLE',0x9108)
GL_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910B)
GL_SAMPLE_MASK=_C('GL_SAMPLE_MASK',0x8E51)
GL_SAMPLE_MASK_VALUE=_C('GL_SAMPLE_MASK_VALUE',0x8E52)
GL_SAMPLE_POSITION=_C('GL_SAMPLE_POSITION',0x8E50)
GL_SIGNALED=_C('GL_SIGNALED',0x9119)
GL_SYNC_CONDITION=_C('GL_SYNC_CONDITION',0x9113)
GL_SYNC_FENCE=_C('GL_SYNC_FENCE',0x9116)
GL_SYNC_FLAGS=_C('GL_SYNC_FLAGS',0x9115)
GL_SYNC_FLUSH_COMMANDS_BIT=_C('GL_SYNC_FLUSH_COMMANDS_BIT',0x00000001)
GL_SYNC_GPU_COMMANDS_COMPLETE=_C('GL_SYNC_GPU_COMMANDS_COMPLETE',0x9117)
GL_SYNC_STATUS=_C('GL_SYNC_STATUS',0x9114)
GL_TEXTURE_2D_MULTISAMPLE=_C('GL_TEXTURE_2D_MULTISAMPLE',0x9100)
GL_TEXTURE_2D_MULTISAMPLE_ARRAY=_C('GL_TEXTURE_2D_MULTISAMPLE_ARRAY',0x9102)
GL_TEXTURE_BINDING_2D_MULTISAMPLE=_C('GL_TEXTURE_BINDING_2D_MULTISAMPLE',0x9104)
GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY=_C('GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY',0x9105)
GL_TEXTURE_CUBE_MAP_SEAMLESS=_C('GL_TEXTURE_CUBE_MAP_SEAMLESS',0x884F)
GL_TEXTURE_FIXED_SAMPLE_LOCATIONS=_C('GL_TEXTURE_FIXED_SAMPLE_LOCATIONS',0x9107)
GL_TEXTURE_SAMPLES=_C('GL_TEXTURE_SAMPLES',0x9106)
GL_TIMEOUT_EXPIRED=_C('GL_TIMEOUT_EXPIRED',0x911B)
GL_TIMEOUT_IGNORED=_C('GL_TIMEOUT_IGNORED',0xFFFFFFFFFFFFFFFF)
GL_TRIANGLES_ADJACENCY=_C('GL_TRIANGLES_ADJACENCY',0x000C)
GL_TRIANGLE_STRIP_ADJACENCY=_C('GL_TRIANGLE_STRIP_ADJACENCY',0x000D)
GL_UNSIGNALED=_C('GL_UNSIGNALED',0x9118)
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE=_C('GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE',0x910A)
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY=_C('GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY',0x910D)
GL_WAIT_FAILED=_C('GL_WAIT_FAILED',0x911D)
@_f
@_p.types(_cs.GLenum,_cs.GLsync,_cs.GLbitfield,_cs.GLuint64)
def glClientWaitSync(sync,flags,timeout):pass
@_f
@_p.types(None,_cs.GLsync)
def glDeleteSync(sync):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLint)
def glDrawElementsBaseVertex(mode,count,type,indices,basevertex):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLsizei,_cs.GLint)
def glDrawElementsInstancedBaseVertex(mode,count,type,indices,instancecount,basevertex):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,_cs.GLenum,ctypes.c_void_p,_cs.GLint)
def glDrawRangeElementsBaseVertex(mode,start,end,count,type,indices,basevertex):pass
@_f
@_p.types(_cs.GLsync,_cs.GLenum,_cs.GLbitfield)
def glFenceSync(condition,flags):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glFramebufferTexture(target,attachment,texture,level):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLint64Array)
def glGetBufferParameteri64v(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLint64Array)
def glGetInteger64i_v(target,index,data):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLint64Array)
def glGetInteger64v(pname,data):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,arrays.GLfloatArray)
def glGetMultisamplefv(pname,index,val):pass
@_f
@_p.types(None,_cs.GLsync,_cs.GLenum,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLintArray)
def glGetSynciv(sync,pname,bufSize,length,values):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLsync)
def glIsSync(sync):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLsizeiArray,_cs.GLenum,arrays.GLvoidpArray,_cs.GLsizei,arrays.GLintArray)
def glMultiDrawElementsBaseVertex(mode,count,type,indices,drawcount,basevertex):pass
@_f
@_p.types(None,_cs.GLenum)
def glProvokingVertex(mode):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLbitfield)
def glSampleMaski(maskNumber,mask):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexImage2DMultisample(target,samples,internalformat,width,height,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexImage3DMultisample(target,samples,internalformat,width,height,depth,fixedsamplelocations):pass
@_f
@_p.types(None,_cs.GLsync,_cs.GLbitfield,_cs.GLuint64)
def glWaitSync(sync,flags,timeout):pass



'''OpenGL extension EXT.visual_rating

This module customises the behaviour of the 
OpenGL.raw.GLX.EXT.visual_rating to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows servers to identify a particular GLX visual as 
	undesirable. A new visual attribute is introduced, providing a way
	for servers to specify caveats (e.g., slow) for a visual. The attribute 
	may be queried using glXGetConfig, and can therefore be used by 
	application-specific visual selection routines. The new attribute is 
	also used by glXChooseVisual to discriminate against visuals with
	caveats.
	
	This extension allows servers to export visuals with improved features
	or image quality, but lower performance or greater system burden,
	without having to have these visuals selected preferentially.  It is
	intended to insure that most applications get the "right" visual, not
	that all applications do.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/visual_rating.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.EXT.visual_rating import *
from OpenGL.raw.GLX.EXT.visual_rating import _EXTENSION_NAME

def glInitVisualRatingEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.map_buffer_alignment

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.map_buffer_alignment to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a requirement to the pointer returned by MapBuffer
	and MapBufferRange that they provide a minimum of 64 byte alignment to
	support processing of the data directly with special CPU instructions
	like SSE and AVX.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/map_buffer_alignment.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.map_buffer_alignment import *
from OpenGL.raw.GL.ARB.map_buffer_alignment import _EXTENSION_NAME

def glInitMapBufferAlignmentARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.texgen_emboss

This module customises the behaviour of the 
OpenGL.raw.GL.NV.texgen_emboss to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a new texture coordinate generation mode
	suitable for multitexture-based embossing (or bump mapping) effects.
	
	Given two texture units, this extension generates the texture
	coordinates of a second texture unit (an odd-numbered texture unit)
	as a perturbation of a first texture unit (an even-numbered texture
	unit one less than the second texture unit).  The perturbation is
	based on the normal, tangent, and light vectors.  The normal vector
	is supplied by glNormal; the light vector is supplied as a direction
	vector to a specified OpenGL light's position; and the tanget
	vector is supplied by the second texture unit's current texture
	coordinate.  The perturbation is also scaled by program-supplied
	scaling constants.
	
	If both texture units are bound to the same texture representing a
	height field, by subtracting the difference between the resulting two
	filtered texels, programs can achieve a per-pixel embossing effect.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texgen_emboss.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.texgen_emboss import *
from OpenGL.raw.GL.NV.texgen_emboss import _EXTENSION_NAME

def glInitTexgenEmbossNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.vertex_array_range2

This module customises the behaviour of the 
OpenGL.raw.GL.NV.vertex_array_range2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	Enabling and disabling the vertex array range is specified by the
	original NV_vertex_array_range extension specification to flush the
	vertex array range implicitly.  In retrospect, this semantic is
	extremely misconceived and creates terrible performance problems
	for any application that wishes to mix conventional vertex arrays
	with vertex arrange range-enabled vertex arrays.
	
	This extension provides a new token for enabling/disabling the
	vertex array range that does NOT perform an implicit vertex array
	range flush when the enable/disable is performed.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/vertex_array_range2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.vertex_array_range2 import *
from OpenGL.raw.GL.NV.vertex_array_range2 import _EXTENSION_NAME

def glInitVertexArrayRange2NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension SGIS.texture_border_clamp

This module customises the behaviour of the 
OpenGL.raw.GL.SGIS.texture_border_clamp to provide a more 
Python-friendly API

Overview (from the spec)
	
	The base OpenGL provides clamping such that the texture coordinates are
	limited to exactly the range [0,1].  When a texture coordinate is
	clamped using this algorithm, the texture sampling filter straddles the
	edge of the texture image, taking 1/2 its sample values from within the
	texture image, and the other 1/2 from the texture border.  It is
	sometimes desirable for a texture to be clamped to the border color,
	rather than to an average of the border and edge colors.
	
	This extension defines an additional texture clamping algorithm.
	CLAMP_TO_BORDER_SGIS clamps texture coordinates at all mipmap levels
	such that NEAREST and LINEAR filters return the color of the border
	texels.  When used with FILTER4 filters, the filter operation of
	CLAMP_TO_BORDER_SGIS is defined but doesn't result in a nice
	clamp-to-border color.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIS/texture_border_clamp.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SGIS.texture_border_clamp import *
from OpenGL.raw.GL.SGIS.texture_border_clamp import _EXTENSION_NAME

def glInitTextureBorderClampSGIS():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_parameter_buffer_object'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_parameter_buffer_object',error_checker=_errors._error_checker)
GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV=_C('GL_FRAGMENT_PROGRAM_PARAMETER_BUFFER_NV',0x8DA4)
GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV=_C('GL_GEOMETRY_PROGRAM_PARAMETER_BUFFER_NV',0x8DA3)
GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV=_C('GL_MAX_PROGRAM_PARAMETER_BUFFER_BINDINGS_NV',0x8DA0)
GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV=_C('GL_MAX_PROGRAM_PARAMETER_BUFFER_SIZE_NV',0x8DA1)
GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV=_C('GL_VERTEX_PROGRAM_PARAMETER_BUFFER_NV',0x8DA2)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLintArray)
def glProgramBufferParametersIivNV(target,bindingIndex,wordIndex,count,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray)
def glProgramBufferParametersIuivNV(target,bindingIndex,wordIndex,count,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLfloatArray)
def glProgramBufferParametersfvNV(target,bindingIndex,wordIndex,count,params):pass



'''OpenGL extension EXT.x11_sync_object

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.x11_sync_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	Synchronization objects added the ability to better coordinate
	operations between multiple GL command streams.  However, it is
	desirable to have the same level of coordination between GL
	command streams and external rendering APIs.  This extension
	introduces two new concepts to build upon the synchronization
	infrastructure provided by ARB_sync:
	
	  1) A means to import an X Synchronization Fence object into the
	     GL and use it as a sync object.
	
	  2) The concept of a reusable sync object.
	
	The latter is necessary because the import operation is expensive
	and performing it every time a synchronization point was reached
	would make the synchronization prohibitively slow.
	
	This extension stops short of allowing the GL to change the state
	of imported/reusable sync objects, but does not add any language
	that would prohibit such functionality from being added in a
	subsequent extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/x11_sync_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.x11_sync_object import *
from OpenGL.raw.GL.EXT.x11_sync_object import _EXTENSION_NAME

def glInitX11SyncObjectEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_MESA_query_renderer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_MESA_query_renderer',error_checker=_errors._error_checker)
GLX_RENDERER_ACCELERATED_MESA=_C('GLX_RENDERER_ACCELERATED_MESA',0x8186)
GLX_RENDERER_DEVICE_ID_MESA=_C('GLX_RENDERER_DEVICE_ID_MESA',0x8184)
GLX_RENDERER_ID_MESA=_C('GLX_RENDERER_ID_MESA',0x818E)
GLX_RENDERER_OPENGL_COMPATIBILITY_PROFILE_VERSION_MESA=_C('GLX_RENDERER_OPENGL_COMPATIBILITY_PROFILE_VERSION_MESA',0x818B)
GLX_RENDERER_OPENGL_CORE_PROFILE_VERSION_MESA=_C('GLX_RENDERER_OPENGL_CORE_PROFILE_VERSION_MESA',0x818A)
GLX_RENDERER_OPENGL_ES2_PROFILE_VERSION_MESA=_C('GLX_RENDERER_OPENGL_ES2_PROFILE_VERSION_MESA',0x818D)
GLX_RENDERER_OPENGL_ES_PROFILE_VERSION_MESA=_C('GLX_RENDERER_OPENGL_ES_PROFILE_VERSION_MESA',0x818C)
GLX_RENDERER_PREFERRED_PROFILE_MESA=_C('GLX_RENDERER_PREFERRED_PROFILE_MESA',0x8189)
GLX_RENDERER_UNIFIED_MEMORY_ARCHITECTURE_MESA=_C('GLX_RENDERER_UNIFIED_MEMORY_ARCHITECTURE_MESA',0x8188)
GLX_RENDERER_VENDOR_ID_MESA=_C('GLX_RENDERER_VENDOR_ID_MESA',0x8183)
GLX_RENDERER_VERSION_MESA=_C('GLX_RENDERER_VERSION_MESA',0x8185)
GLX_RENDERER_VIDEO_MEMORY_MESA=_C('GLX_RENDERER_VIDEO_MEMORY_MESA',0x8187)
@_f
@_p.types(_cs.Bool,_cs.c_int,ctypes.POINTER(_cs.c_uint))
def glXQueryCurrentRendererIntegerMESA(attribute,value):pass
@_f
@_p.types(ctypes.c_char_p,_cs.c_int)
def glXQueryCurrentRendererStringMESA(attribute):pass
@_f
@_p.types(_cs.Bool,ctypes.POINTER(_cs.Display),_cs.c_int,_cs.c_int,_cs.c_int,ctypes.POINTER(_cs.c_uint))
def glXQueryRendererIntegerMESA(dpy,screen,renderer,attribute,value):pass
@_f
@_p.types(ctypes.c_char_p,ctypes.POINTER(_cs.Display),_cs.c_int,_cs.c_int,_cs.c_int)
def glXQueryRendererStringMESA(dpy,screen,renderer,attribute):pass



'''OpenGL extension APPLE.vertex_array_range

This module customises the behaviour of the 
OpenGL.raw.GL.APPLE.vertex_array_range to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension is designed to allow very high vertex processing rates which
	are facilitated both by relieving the CPU of as much processing burden as
	possible and by allowing graphics hardware to directly access vertex data.
	Because this extension is implemented as an addition to the vertex array
	specification provided by OpenGL 1.1, applications can continue to use
	existing vertex submission logic while taking advantage of vertex array
	ranges to more efficiently process those arrays.
	
	The vertex array coherency model provided by OpenGL 1.1 requires that
	vertex data specified in vertex arrays be transferred from system memory
	each time Begin, DrawArrays, or DrawElements is called.  Further, OpenGL
	1.1 requires that the transfer of data be completed by the time End,
	DrawArrays, or DrawElements returns.  Both of these requirements are
	relaxed by the vertex array range extension.  Vertex data may be cached
	by the GL so there is no guarantee that changes to the vertex data will
	be reflected in following drawing commands unless it is flushed with
	FlushVertexArrayRangeAPPLE.  The reading of vertex data may be deferred 
	by the GL so there is no guarantee that the GL will be finished reading
	the data until completion is forced by the use of Finish or the APPLE_fence
	extension.
	
	Vertex array range can be enabled in two ways.  EnableClientState can be
	used with the VERTEX_ARRAY_RANGE_APPLE param to enable vertex array range
	for the client context.  One can also simply set the vertex array storage
	hint to either STORAGE_CACHED_APPLE or STORAGE_SHARED_APPLE (as discussed
	below) to enable a particular vertex array range.  Once this is done, use of
	vertex array range requires the definition of a specific memory range for
	vertex data through VertexArrayRangeAPPLE.  It is recommended this data be
	page aligned (4096 byte boundaries) and a multiple of page size in length
	for maximum efficiency in data handling and internal flushing, but this is
	not a requirement and any location and length of data can be defined as a
	vertex array.  This extension provides no memory allocators as any
	convenient memory allocator can be used.
	
	Once a data set is established, using VertexArrayRangeAPPLE, it can be can
	be drawn using standard OpenGL vertex array commands, as one would do
	without this extension.  Note, if any the data for any enabled array for a
	given array element index falls outside of the vertex array range, an
	undefined vertex is generated.  One should also understand removing or
	replacing all calls to vertex array range functions with no-ops or disabling
	the vertex array range by disabling the VERTEX_ARRAY_RANGE_APPLE client
	state should not change the results of an application's OpenGL drawing.
	
	For static data no additional coherency nor synchronization must be done and
	the client is free to draw with the specified draw as it sees fit.
	
	If data is dynamic, thus to be modified, FlushVertexArrayRangeAPPLE should
	be used.  The command is issued when data has been modified since the last
	call to VertexArrayRangeAPPLE or FlushVertexArrayRangeAPPLE and prior to
	drawing with such data. FlushVertexArrayRangeAPPLE only provides memory
	coherency prior to drawing (such as ensuring CPU caches are flushed or VRAM
	cached copies are updated) and does not provide any synchronization with
	previously issued drawing commands. The range flushed can be the specific
	range modified and does not have to be the entire vertex array range.
	Additionally, data maybe read immediately after a flush without need for
	further synchronization, thus overlapping areas of data maybe read, modified
	and written between two successive flushes and the data will be
	consistent.
	
	To synchronize data modification after drawing two methods can be used. A
	Finish command can be issued which will not return until all previously
	issued commands are complete, forcing completely synchronous operation.
	While this guarantees all drawing is complete it may not be the optimal
	solution for clients which just need to ensure drawing with the vertex array
	range or a specific range with the array is compete.  The APPLE_fence
	extension can be used when dynamic data modifications need to be
	synchronized with drawing commands. Specifically, if data is to be modified,
	a fence can be set immediately after drawing with the data.  Once it comes
	time to modify the data, the application must test (or finish) this fence to
	ensure the drawing command has completed. Failure to do this could result in
	new data being used by the previously issued drawing commands.  It should be
	noted that providing the maximum time between the drawing set fence and the
	modification test/finish fence allows the most asynchronous behavior and
	will result in the least stalling waiting for drawing completion. Techniques
	such as double buffering vertex data can be used to help further prevent
	stalls based on fence completion but are beyond the scope of this extension.
	
	Once an application is finished with a specific vertex array range or at
	latest prior to exit, and prior to freeing the memory associated with this
	vertex array, the client should call VertexArrayRangeAPPLE with a data
	location and length of 0 to allow the internal memory managers to complete
	any commitments for the array range.  In this case once
	VertexArrayRangeAPPLE returns it is safe to de-allocate the memory.
	
	Three types of storage hints are available for vertex array ranges; client,
	shared, and cached.  These hints are set by passing the
	STORAGE_CLIENT_APPLE, STORAGE_SHARED_APPLE, or STORAGE_CACHED_APPLE param to
	VertexArrayParameteriAPPLE with VERTEX_ARRAY_STORAGE_HINT_APPLE pname.
	Client storage, the default OpenGL behavior, occurs when
	VERTEX_ARRAY_RANGE_APPLE is disabled AND the STORAGE_CLIENT_APPLE hint is
	set. Note, STORAGE_CLIENT_APPLE is also the default hint setting.  Shared
	memory usage is normally used for dynamic data that is expected to be
	modified and is likely mapped to AGP memory space for access by both the
	graphics hardware and client.  It is set when either
	VERTEX_ARRAY_RANGE_APPLE is enabled, without the STORAGE_CACHED_APPLE hint
	being set, or in all cases when the STORAGE_SHARED_APPLE hint is set.
	Finally, the cached storage is designed to support static data and data which
	could be cached in VRAM. This provides maximum access bandwidth for the
	vertex array and occurs when the STORAGE_CACHED_APPLE hint is set. 
	
	The following pseudo-code represents the treatment of a vertex array range
	memory depending on the hint setting and whether vertex array range is
	enabled for the client context:
	
	if (VERTEX_ARRAY_STORAGE_HINT_APPLE == STORAGE_CACHED_APPLE)
	    vertex array is treated as cached
	else if (VERTEX_ARRAY_STORAGE_HINT_APPLE == STORAGE_SHARED_APPLE)
		vertex array is treated as shared
	else if (VERTEX_ARRAY_RANGE_APPLE enabled)
		vertex array is treated as shared
	else 
		vertex array is treated as client
	
	Note, these hints can affect how array flushes are handled and the overhead
	associated with flushing the array, it is recommended that data be handled
	as shared unless it really is static and there are no plans to modify it.
	
	To summarize the vertex array range extension provides relaxed
	synchronization rules for handling vertex array data allowing high bandwidth
	asynchronous data transfer from client memory to graphics hardware.
	Different flushing and synchronization rules are required to ensure data
	coherency when modifying data.  Lastly, memory handling hints are provided
	to allow the tunning of memory storage and access for maximum efficiency.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/APPLE/vertex_array_range.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.APPLE.vertex_array_range import *
from OpenGL.raw.GL.APPLE.vertex_array_range import _EXTENSION_NAME

def glInitVertexArrayRangeAPPLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexArrayRangeAPPLE=wrapper.wrapper(glVertexArrayRangeAPPLE).setOutput(
    'pointer',size=lambda x:(x,),pnameArg='length',orPassIn=True
)
glFlushVertexArrayRangeAPPLE=wrapper.wrapper(glFlushVertexArrayRangeAPPLE).setOutput(
    'pointer',size=lambda x:(x,),pnameArg='length',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension SGIX.pbuffer

This module customises the behaviour of the 
OpenGL.raw.GLX.SGIX.pbuffer to provide a more 
Python-friendly API

Overview (from the spec)
	
	    This extension defines pixel buffers (GLXPbuffers, or pbuffer for
	    short). GLXPbuffers are additional non-visible rendering buffers for an
	    OpenGL renderer.  GLXPbuffers are equivalent to GLXPixmaps with the
	    following exceptions:
	
	    1.	There is no associated X pixmap. Also, since a GLXPbuffer is a GLX
	        resource, it may not be possible to render to it using X or an 
	   	X extension other than GLX.
	
	    2.	The format of the color buffers and the type and size of any
	     	associated ancillary buffers for a GLXPbuffer can only be
	        described with a GLXFBConfig -- an X Visual cannot be used.
	
	    3.	It is possible to create a GLXPbuffer whose contents may be 
		asynchronously lost at any time.
	
	    4.  GLXPbuffers can be rendered to using either direct or indirect
	        rendering contexts.
	
	    5.  The allocation of a GLXPbuffer can fail if there are insufficient
		resources (i.e., all the pbuffer memory has been allocated and 
		the implementation does not virtualize pbuffer memory.)
	
	    The intent of the pbuffer semantics is to enable implementations to
	    allocate pbuffers in non-visible frame buffer memory.  These
	    pbuffers are intended to be "static" resources, in that a program
	    will typically allocate them only once, rather than as a part of its
	    rendering loop.  (But they should be deallocated when the program is
	    no longer using them -- for example, if the program is iconified.)
	    The frame buffer resources that are associated with a pbuffer are 
	    also static, and are deallocated only when the pbuffer is destroyed, 
	    or, in the case of a "unpreserved" pbuffer, as a result of X server 
	    activity that changes its frame buffer requirements.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGIX/pbuffer.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.SGIX.pbuffer import *
from OpenGL.raw.GLX.SGIX.pbuffer import _EXTENSION_NAME

def glInitPbufferSGIX():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_gpu_shader_int64'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_gpu_shader_int64',error_checker=_errors._error_checker)
GL_FLOAT16_NV=_C('GL_FLOAT16_NV',0x8FF8)
GL_FLOAT16_VEC2_NV=_C('GL_FLOAT16_VEC2_NV',0x8FF9)
GL_FLOAT16_VEC3_NV=_C('GL_FLOAT16_VEC3_NV',0x8FFA)
GL_FLOAT16_VEC4_NV=_C('GL_FLOAT16_VEC4_NV',0x8FFB)
GL_INT16_NV=_C('GL_INT16_NV',0x8FE4)
GL_INT16_VEC2_NV=_C('GL_INT16_VEC2_NV',0x8FE5)
GL_INT16_VEC3_NV=_C('GL_INT16_VEC3_NV',0x8FE6)
GL_INT16_VEC4_NV=_C('GL_INT16_VEC4_NV',0x8FE7)
GL_INT64_NV=_C('GL_INT64_NV',0x140E)
GL_INT64_VEC2_NV=_C('GL_INT64_VEC2_NV',0x8FE9)
GL_INT64_VEC3_NV=_C('GL_INT64_VEC3_NV',0x8FEA)
GL_INT64_VEC4_NV=_C('GL_INT64_VEC4_NV',0x8FEB)
GL_INT8_NV=_C('GL_INT8_NV',0x8FE0)
GL_INT8_VEC2_NV=_C('GL_INT8_VEC2_NV',0x8FE1)
GL_INT8_VEC3_NV=_C('GL_INT8_VEC3_NV',0x8FE2)
GL_INT8_VEC4_NV=_C('GL_INT8_VEC4_NV',0x8FE3)
GL_UNSIGNED_INT16_NV=_C('GL_UNSIGNED_INT16_NV',0x8FF0)
GL_UNSIGNED_INT16_VEC2_NV=_C('GL_UNSIGNED_INT16_VEC2_NV',0x8FF1)
GL_UNSIGNED_INT16_VEC3_NV=_C('GL_UNSIGNED_INT16_VEC3_NV',0x8FF2)
GL_UNSIGNED_INT16_VEC4_NV=_C('GL_UNSIGNED_INT16_VEC4_NV',0x8FF3)
GL_UNSIGNED_INT64_NV=_C('GL_UNSIGNED_INT64_NV',0x140F)
GL_UNSIGNED_INT64_VEC2_NV=_C('GL_UNSIGNED_INT64_VEC2_NV',0x8FF5)
GL_UNSIGNED_INT64_VEC3_NV=_C('GL_UNSIGNED_INT64_VEC3_NV',0x8FF6)
GL_UNSIGNED_INT64_VEC4_NV=_C('GL_UNSIGNED_INT64_VEC4_NV',0x8FF7)
GL_UNSIGNED_INT8_NV=_C('GL_UNSIGNED_INT8_NV',0x8FEC)
GL_UNSIGNED_INT8_VEC2_NV=_C('GL_UNSIGNED_INT8_VEC2_NV',0x8FED)
GL_UNSIGNED_INT8_VEC3_NV=_C('GL_UNSIGNED_INT8_VEC3_NV',0x8FEE)
GL_UNSIGNED_INT8_VEC4_NV=_C('GL_UNSIGNED_INT8_VEC4_NV',0x8FEF)
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLint64Array)
def glGetUniformi64vNV(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLuint64Array)
def glGetUniformui64vNV(program,location,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64EXT)
def glProgramUniform1i64NV(program,location,x):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform1i64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64EXT)
def glProgramUniform1ui64NV(program,location,x):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform1ui64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT)
def glProgramUniform2i64NV(program,location,x,y):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform2i64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glProgramUniform2ui64NV(program,location,x,y):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform2ui64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT)
def glProgramUniform3i64NV(program,location,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform3i64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glProgramUniform3ui64NV(program,location,x,y,z):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform3ui64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT)
def glProgramUniform4i64NV(program,location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glProgramUniform4i64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glProgramUniform4ui64NV(program,location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniform4ui64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64EXT)
def glUniform1i64NV(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform1i64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64EXT)
def glUniform1ui64NV(location,x):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform1ui64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT)
def glUniform2i64NV(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform2i64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glUniform2ui64NV(location,x,y):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform2ui64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT)
def glUniform3i64NV(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform3i64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glUniform3ui64NV(location,x,y,z):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform3ui64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT,_cs.GLint64EXT)
def glUniform4i64NV(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLint64Array)
def glUniform4i64vNV(location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT,_cs.GLuint64EXT)
def glUniform4ui64NV(location,x,y,z,w):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniform4ui64vNV(location,count,value):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_QCOM_extended_get2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_QCOM_extended_get2',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLcharArray,arrays.GLintArray)
def glExtGetProgramBinarySourceQCOM(program,shadertype,source,length):pass
@_f
@_p.types(None,arrays.GLuintArray,_cs.GLint,arrays.GLintArray)
def glExtGetProgramsQCOM(programs,maxPrograms,numPrograms):pass
@_f
@_p.types(None,arrays.GLuintArray,_cs.GLint,arrays.GLintArray)
def glExtGetShadersQCOM(shaders,maxShaders,numShaders):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glExtIsProgramBinaryQCOM(program):pass



"""Run-time calculation of offset into Python string structure

Does a scan to find the digits of pi in a string structure
in order to produce an offset that can be used to produce 
data-pointers from Python strings.

Porting note:
    
    Currently this uses id( str a ) to get the base address
    of the Python string.  Python implementations where id( a )
    is *not* the memory address of the string will not work!
"""
import ctypes
from OpenGL._bytes import bytes,unicode
PI_DIGITS = '31415926535897931'

def calculateOffset( ):
    """Calculates the data-pointer offset for strings
    
    This does a sequential scan for 100 bytes from the id
    of a string to find special data-value stored in the
    string (the digits of PI).  It produces a dataPointer
    function which adds that offset to the id of the 
    passed strings.
    """
    finalOffset = None
    a = PI_DIGITS
    # XXX NOT portable across Python implmentations!!!
    initial = id(a)
    targetType = ctypes.POINTER( ctypes.c_char )
    for offset in range( 100 ):
        vector = ctypes.cast( initial+offset,targetType )
        allMatched = True
        for index,digit in enumerate( a ):
            if vector[index] != digit:
                allMatched = False
                break
        if allMatched:
            finalOffset = offset
            break
    if finalOffset is not None:
        def dataPointer( data ):
            """Return the data-pointer from the array using calculated offset
            
            data -- a Python string
            
            Returns the raw data-pointer to the internal buffer of the passed string
            """
            if not isinstance( data, bytes ):
                raise TypeError(
                    """This function can only handle Python strings!  Got %s"""%(
                        type(data),
                    )
                )
            return id(data) + finalOffset
        # just for later reference...
        dataPointer.offset = finalOffset 
        return dataPointer
    raise RuntimeError(
        """Unable to determine dataPointer offset for strings!"""
    )

dataPointer = calculateOffset()

if __name__ == "__main__":
    a  = 'this'
    print((id(a), dataPointer( a ), dataPointer(a) - id(a)))
    



'''OpenGL extension ARB.derivative_control

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.derivative_control to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides control over the spacial granularity at which the
	underlying implementation computes derivatives.  
	
	For example, for the coarse-granularity derivative, a single x derivative
	could be computed for each 2x2 group of pixels, using that same derivative
	value for all 4 pixels.  For the fine-granularity derivative, two
	derivatives could be computed for each 2x2 group of pixels; one for the top
	row and one for the bottom row.  Implementations vary somewhat on how this
	is done.
	
	To select the coarse derivative, use:
	
	    dFdxCoarse(p)
	    dFdyCoarse(p)
	    fwidthCoarse(p)
	
	To select the fine derivative, use:
	
	    dFdxFine(p)
	    dFdyFine(p)
	    fwidthFine(p)
	
	To select which ever is "better" (based on performance, API hints, or other
	factors), use:
	
	    dFdx(p)
	    dFdy(p)
	    fwidth(p)
	
	This last set is the set of previously existing built-ins for derivatives,
	and continues to work in a backward compatible way.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/derivative_control.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.derivative_control import *
from OpenGL.raw.GL.ARB.derivative_control import _EXTENSION_NAME

def glInitDerivativeControlARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.program_interface_query

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.program_interface_query to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a single unified set of query commands that can be
	used by applications to determine properties of various interfaces and
	resources used by program objects to communicate with application code,
	fixed-function OpenGL pipeline stages, and other programs.  In unextended
	OpenGL 4.2, there is a separate set of query commands for each different
	type of interface or resource used by the program.  These different sets
	of queries are structured nearly identically, but the queries for some
	interfaces have limited capability (e.g., there is no ability to enumerate
	fragment shader outputs).
	
	With the single set of query commands provided by this extension, a
	consistent set of queries is available for all interfaces, and a new
	interface can be added without having to introduce a completely new set of
	query commands.  These queries are intended to provide a superset of the
	capabilities provided by similar queries in OpenGL 4.2, and should allow
	for the deprecation of the existing queries.
	
	This extension defines two terms:  interfaces and active resources.  Each
	interface of a program object provides a way for the program to
	communicate with application code, fixed-function OpenGL pipeline stages,
	and other programs.  Examples of interfaces for a program object include
	inputs (receiving values from vertex attributes or outputs of other
	programs), outputs (sending values to other programs or per-fragment
	operations), uniforms (receiving values from API calls), uniform blocks
	(receiving values from bound buffer objects), subroutines and subroutine
	uniforms (receiving API calls to indicate functions to call during program
	execution), and atomic counter buffers (holding values to be manipulated
	by atomic counter shader functions).  Each interface of a program has a
	set of active resources used by the program.  For example, the resources
	of a program's input interface includes all active input variables used by
	the first stage of the program.  The resources of a program's uniform
	block interface consists of the set of uniform blocks with at least one
	member used by any shader in the program.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/program_interface_query.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.program_interface_query import *
from OpenGL.raw.GL.ARB.program_interface_query import _EXTENSION_NAME

def glInitProgramInterfaceQueryARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetProgramInterfaceiv=wrapper.wrapper(glGetProgramInterfaceiv).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glGetProgramResourceIndex.name size not checked against 'name'
glGetProgramResourceIndex=wrapper.wrapper(glGetProgramResourceIndex).setInputArraySize(
    'name', None
)
glGetProgramResourceName=wrapper.wrapper(glGetProgramResourceName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
# INPUT glGetProgramResourceiv.props size not checked against propCount
glGetProgramResourceiv=wrapper.wrapper(glGetProgramResourceiv).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
).setInputArraySize(
    'props', None
)
# INPUT glGetProgramResourceLocation.name size not checked against 'name'
glGetProgramResourceLocation=wrapper.wrapper(glGetProgramResourceLocation).setInputArraySize(
    'name', None
)
# INPUT glGetProgramResourceLocationIndex.name size not checked against 'name'
glGetProgramResourceLocationIndex=wrapper.wrapper(glGetProgramResourceLocationIndex).setInputArraySize(
    'name', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_texture_buffer_range'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_texture_buffer_range',error_checker=_errors._error_checker)
GL_TEXTURE_BUFFER_OFFSET=_C('GL_TEXTURE_BUFFER_OFFSET',0x919D)
GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT=_C('GL_TEXTURE_BUFFER_OFFSET_ALIGNMENT',0x919F)
GL_TEXTURE_BUFFER_SIZE=_C('GL_TEXTURE_BUFFER_SIZE',0x919E)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr)
def glTexBufferRange(target,internalformat,buffer,offset,size):pass



'''OpenGL extension ARB.shader_subroutine

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.shader_subroutine to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support to shaders for "indirect subroutine calls", 
	where a single shader can include many subroutines and dynamically select
	through the API which subroutine is called from each call site. 
	Switching subroutines dynamically in this fashion can avoid the cost of 
	recompiling and managing multiple shaders, while still retaining most of
	the performance of specialized shaders.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/shader_subroutine.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.shader_subroutine import *
from OpenGL.raw.GL.ARB.shader_subroutine import _EXTENSION_NAME

def glInitShaderSubroutineARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetActiveSubroutineUniformiv=wrapper.wrapper(glGetActiveSubroutineUniformiv).setOutput(
    'values',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetActiveSubroutineUniformName=wrapper.wrapper(glGetActiveSubroutineUniformName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufsize',orPassIn=True
)
glGetActiveSubroutineName=wrapper.wrapper(glGetActiveSubroutineName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'name',size=lambda x:(x,),pnameArg='bufsize',orPassIn=True
)
# INPUT glUniformSubroutinesuiv.indices size not checked against count
glUniformSubroutinesuiv=wrapper.wrapper(glUniformSubroutinesuiv).setInputArraySize(
    'indices', None
)
glGetUniformSubroutineuiv=wrapper.wrapper(glGetUniformSubroutineuiv).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetProgramStageiv=wrapper.wrapper(glGetProgramStageiv).setOutput(
    'values',size=(1,),orPassIn=True
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_mapbuffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_mapbuffer',error_checker=_errors._error_checker)
GL_BUFFER_ACCESS_OES=_C('GL_BUFFER_ACCESS_OES',0x88BB)
GL_BUFFER_MAPPED_OES=_C('GL_BUFFER_MAPPED_OES',0x88BC)
GL_BUFFER_MAP_POINTER_OES=_C('GL_BUFFER_MAP_POINTER_OES',0x88BD)
GL_WRITE_ONLY_OES=_C('GL_WRITE_ONLY_OES',0x88B9)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLvoidpArray)
def glGetBufferPointervOES(target,pname,params):pass
@_f
@_p.types(ctypes.c_void_p,_cs.GLenum,_cs.GLenum)
def glMapBufferOES(target,access):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum)
def glUnmapBufferOES(target):pass



'''OpenGL extension EXT.GL_422_pixels

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.GL_422_pixels to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for converting 422 pixels in host
	memory to 444 pixels as part of the pixel storage operation.
	
	The pixel unpack storage operation treats a 422 pixel as a 2 element
	format where the first element is C (chrominance) and the second
	element is L (luminance). Luminance is present on all pixels; a full
	chrominance value requires two pixels.
	
	The pixel pack storage operation converts RGB to a 422 pixel defined as
	a 2 element format where the first element stored is C (chrominance)
	and the second element stored is L (luminance).  Luminance is present
	on all pixels; a full chrominance value requires two pixels.
	
	Both averaging and non-averaging is supported for green and blue
	assignments for pack and unpack operations.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/GL_422_pixels.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.GL_422_pixels import *
from OpenGL.raw.GL.EXT.GL_422_pixels import _EXTENSION_NAME

def glInitGl422PixelsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.gpu_program4

This module customises the behaviour of the 
OpenGL.raw.GL.NV.gpu_program4 to provide a more 
Python-friendly API

Overview (from the spec)
	
	This specification documents the common instruction set and basic
	functionality provided by NVIDIA's 4th generation of assembly instruction
	sets supporting programmable graphics pipeline stages.  
	
	The instruction set builds upon the basic framework provided by the
	ARB_vertex_program and ARB_fragment_program extensions to expose
	considerably more capable hardware.  In addition to new capabilities for
	vertex and fragment programs, this extension provides a new program type
	(geometry programs) further described in the NV_geometry_program4
	specification.
	
	NV_gpu_program4 provides a unified instruction set -- all instruction set
	features are available for all program types, except for a small number of
	features that make sense only for a specific program type.  It provides
	fully capable signed and unsigned integer data types, along with a set of
	arithmetic, logical, and data type conversion instructions capable of
	operating on integers.  It also provides a uniform set of structured
	branching constructs (if tests, loops, and subroutines) that fully support
	run-time condition testing.
	
	This extension provides several new texture mapping capabilities.  Shadow
	cube maps are supported, where cube map faces can encode depth values.
	Texture lookup instructions can include an immediate texel offset, which
	can assist in advanced filtering.  New instructions are provided to fetch
	a single texel by address in a texture map (TXF) and query the size of a
	specified texture level (TXQ).
	
	By and large, vertex and fragment programs written to ARB_vertex_program
	and ARB_fragment_program can be ported directly by simply changing the
	program header from "!!ARBvp1.0" or "!!ARBfp1.0" to "!!NVvp4.0" or
	"!!NVfp4.0", and then modifying the code to take advantage of the expanded
	feature set.  There are a small number of areas where this extension is
	not a functional superset of previous vertex program extensions, which are
	documented in this specification.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/gpu_program4.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.gpu_program4 import *
from OpenGL.raw.GL.NV.gpu_program4 import _EXTENSION_NAME

def glInitGpuProgram4NV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glProgramLocalParameterI4ivNV=wrapper.wrapper(glProgramLocalParameterI4ivNV).setInputArraySize(
    'params', 4
)
# INPUT glProgramLocalParametersI4ivNV.params size not checked against count*4
glProgramLocalParametersI4ivNV=wrapper.wrapper(glProgramLocalParametersI4ivNV).setInputArraySize(
    'params', None
)
glProgramLocalParameterI4uivNV=wrapper.wrapper(glProgramLocalParameterI4uivNV).setInputArraySize(
    'params', 4
)
# INPUT glProgramLocalParametersI4uivNV.params size not checked against count*4
glProgramLocalParametersI4uivNV=wrapper.wrapper(glProgramLocalParametersI4uivNV).setInputArraySize(
    'params', None
)
glProgramEnvParameterI4ivNV=wrapper.wrapper(glProgramEnvParameterI4ivNV).setInputArraySize(
    'params', 4
)
# INPUT glProgramEnvParametersI4ivNV.params size not checked against count*4
glProgramEnvParametersI4ivNV=wrapper.wrapper(glProgramEnvParametersI4ivNV).setInputArraySize(
    'params', None
)
glProgramEnvParameterI4uivNV=wrapper.wrapper(glProgramEnvParameterI4uivNV).setInputArraySize(
    'params', 4
)
# INPUT glProgramEnvParametersI4uivNV.params size not checked against count*4
glProgramEnvParametersI4uivNV=wrapper.wrapper(glProgramEnvParametersI4uivNV).setInputArraySize(
    'params', None
)
glGetProgramLocalParameterIivNV=wrapper.wrapper(glGetProgramLocalParameterIivNV).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramLocalParameterIuivNV=wrapper.wrapper(glGetProgramLocalParameterIuivNV).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramEnvParameterIivNV=wrapper.wrapper(glGetProgramEnvParameterIivNV).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetProgramEnvParameterIuivNV=wrapper.wrapper(glGetProgramEnvParameterIuivNV).setOutput(
    'params',size=(4,),orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension ATI.meminfo

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.meminfo to provide a more 
Python-friendly API

Overview (from the spec)
	
	Traditionally, OpenGL has treated resource management as a task of hardware
	virtualization hidden from applications. While providing great portability,
	this shielding of information can prevent applications from making
	intelligent decisions on the management of resources they create. For
	instance, an application may be better served by choosing a different
	rendering method if there is not sufficient resources to efficiently
	utilize its preferred method.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/meminfo.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.meminfo import *
from OpenGL.raw.GL.ATI.meminfo import _EXTENSION_NAME

def glInitMeminfoATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ATI_vertex_attrib_array_object'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ATI_vertex_attrib_array_object',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLfloatArray)
def glGetVertexAttribArrayObjectfvATI(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLintArray)
def glGetVertexAttribArrayObjectivATI(index,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLenum,_cs.GLboolean,_cs.GLsizei,_cs.GLuint,_cs.GLuint)
def glVertexAttribArrayObjectATI(index,size,type,normalized,stride,buffer,offset):pass



'''OpenGL extension EXT.multi_draw_arrays

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.multi_draw_arrays to provide a more 
Python-friendly API

Overview (from the spec)
	
	These functions behave identically to the standard OpenGL 1.1 functions
	glDrawArrays() and glDrawElements() except they handle multiple lists of
	vertices in one call. Their main purpose is to allow one function call
	to render more than one primitive such as triangle strip, triangle fan,
	etc.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/multi_draw_arrays.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.multi_draw_arrays import *
from OpenGL.raw.GL.EXT.multi_draw_arrays import _EXTENSION_NAME

def glInitMultiDrawArraysEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glMultiDrawArraysEXT.count size not checked against 'primcount'
# INPUT glMultiDrawArraysEXT.first size not checked against 'primcount'
glMultiDrawArraysEXT=wrapper.wrapper(glMultiDrawArraysEXT).setInputArraySize(
    'count', None
).setInputArraySize(
    'first', None
)
# INPUT glMultiDrawElementsEXT.count size not checked against 'primcount'
# INPUT glMultiDrawElementsEXT.indices size not checked against 'primcount'
glMultiDrawElementsEXT=wrapper.wrapper(glMultiDrawElementsEXT).setInputArraySize(
    'count', None
).setInputArraySize(
    'indices', None
)
### END AUTOGENERATED SECTION



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLX import _types as _cs
# End users want this...
from OpenGL.raw.GLX._types import *
from OpenGL.raw.GLX import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLX_EXT_visual_info'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLX,'GLX_EXT_visual_info',error_checker=_errors._error_checker)
GLX_DIRECT_COLOR_EXT=_C('GLX_DIRECT_COLOR_EXT',0x8003)
GLX_GRAY_SCALE_EXT=_C('GLX_GRAY_SCALE_EXT',0x8006)
GLX_NONE_EXT=_C('GLX_NONE_EXT',0x8000)
GLX_PSEUDO_COLOR_EXT=_C('GLX_PSEUDO_COLOR_EXT',0x8004)
GLX_STATIC_COLOR_EXT=_C('GLX_STATIC_COLOR_EXT',0x8005)
GLX_STATIC_GRAY_EXT=_C('GLX_STATIC_GRAY_EXT',0x8007)
GLX_TRANSPARENT_ALPHA_VALUE_EXT=_C('GLX_TRANSPARENT_ALPHA_VALUE_EXT',0x28)
GLX_TRANSPARENT_BLUE_VALUE_EXT=_C('GLX_TRANSPARENT_BLUE_VALUE_EXT',0x27)
GLX_TRANSPARENT_GREEN_VALUE_EXT=_C('GLX_TRANSPARENT_GREEN_VALUE_EXT',0x26)
GLX_TRANSPARENT_INDEX_EXT=_C('GLX_TRANSPARENT_INDEX_EXT',0x8009)
GLX_TRANSPARENT_INDEX_VALUE_EXT=_C('GLX_TRANSPARENT_INDEX_VALUE_EXT',0x24)
GLX_TRANSPARENT_RED_VALUE_EXT=_C('GLX_TRANSPARENT_RED_VALUE_EXT',0x25)
GLX_TRANSPARENT_RGB_EXT=_C('GLX_TRANSPARENT_RGB_EXT',0x8008)
GLX_TRANSPARENT_TYPE_EXT=_C('GLX_TRANSPARENT_TYPE_EXT',0x23)
GLX_TRUE_COLOR_EXT=_C('GLX_TRUE_COLOR_EXT',0x8002)
GLX_X_VISUAL_TYPE_EXT=_C('GLX_X_VISUAL_TYPE_EXT',0x22)




'''OpenGL extension EXT.direct_state_access

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.direct_state_access to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a set of new "direct state access"
	commands (meaning no selector is involved) to access (update and
	query) OpenGL state that previously depended on the OpenGL state
	selectors for access.  These new commands supplement the existing
	selector-based OpenGL commands to access the same state.
	
	The intent of this extension is to make it more efficient for
	libraries to avoid disturbing selector and latched state.  The
	extension also allows more efficient command usage by eliminating
	the need for selector update commands.
	
	Two derivative advantages of this extension are 1) display lists
	can be executed using these commands that avoid disturbing selectors
	that subsequent commands may depend on, and 2) drivers implemented
	with a dual-thread partitioning with OpenGL command buffering from
	an application thread and then OpenGL command dispatching in a
	concurrent driver thread can avoid thread synchronization created by
	selector saving, setting, command execution, and selector restoration.
	
	This extension does not itself add any new OpenGL state.
	
	We call a state variable in OpenGL an "OpenGL state selector" or
	simply a "selector" if OpenGL commands depend on the state variable
	to determine what state to query or update.  The matrix mode and
	active texture are both selectors.  Object bindings for buffers,
	programs, textures, and framebuffer objects are also selectors.
	
	We call OpenGL state "latched" if the state is set by one OpenGL
	command but then that state is saved by a subsequent command or the
	state determines how client memory or buffer object memory is accessed
	by a subsequent command.  The array and element array buffer bindings
	are latched by vertex array specification commands to determine
	which buffer a given vertex array uses.  Vertex array state and pixel
	pack/unpack state decides how client memory or buffer object memory is
	accessed by subsequent vertex pulling or image specification commands.
	
	The existence of selectors and latched state in the OpenGL API
	reduces the number of parameters to various sets of OpenGL commands
	but complicates the access to state for layered libraries which seek
	to access state without disturbing other state, namely the state of
	state selectors and latched state.  In many cases, selectors and
	latched state were introduced by extensions as OpenGL evolved to
	minimize the disruption to the OpenGL API when new functionality,
	particularly the pluralization of existing functionality as when
	texture objects and later multiple texture units, was introduced.
	
	The OpenGL API involves several selectors (listed in historical
	order of introduction):
	
	  o  The matrix mode.
	
	  o  The current bound texture for each supported texture target.
	
	  o  The active texture.
	
	  o  The active client texture.
	
	  o  The current bound program for each supported program target.
	
	  o  The current bound buffer for each supported buffer target.
	
	  o  The current GLSL program.
	
	  o  The current framebuffer object.
	
	The new selector-free update commands can be compiled into display
	lists.
	
	The OpenGL API has latched state for vertex array buffer objects
	and pixel store state.  When an application issues a GL command to
	unpack or pack pixels (for example, glTexImage2D or glReadPixels
	respectively), the current unpack and pack pixel store state
	determines how the pixels are unpacked from/packed to client memory
	or pixel buffer objects.  For example, consider:
	
	  glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_TRUE);
	  glPixelStorei(GL_UNPACK_ROW_LENGTH, 640);
	  glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 47);
	  glDrawPixels(100, 100, GL_RGB, GL_FLOAT, pixels);
	
	The unpack swap bytes and row length state set by the preceding
	glPixelStorei commands (as well as the 6 other unpack pixel store
	state variables) control how data is read (unpacked) from buffer of
	data pointed to by pixels.  The glBindBuffer command also specifies
	an unpack buffer object (47) so the pixel pointer is actually treated
	as a byte offset into buffer object 47.
	
	When an application issues a command to configure a vertex array,
	the current array buffer state is latched as the binding for the
	particular vertex array being specified.  For example, consider:
	
	  glBindBuffer(GL_ARRAY_BUFFER, 23);
	  glVertexPointer(3, GL_FLOAT, 12, pointer);
	
	The glBindBuffer command updates the array buffering binding
	(GL_ARRAY_BUFFER_BINDING) to the buffer object named 23.  The
	subsequent glVertexPointer command specifies explicit parameters
	for the size, type, stride, and pointer to access the position
	vertex array BUT ALSO latches the current array buffer binding for
	the vertex array buffer binding (GL_VERTEX_ARRAY_BUFFER_BINDING).
	Effectively the current array buffer binding buffer object becomes
	an implicit fifth parameter to glVertexPointer and this applies to
	all the gl*Pointer vertex array specification commands.
	
	Selectors and latched state create problems for layered libraries
	using OpenGL because selectors require the selector state to be
	modified to update some other state and latched state means implicit
	state can affect the operation of commands specifying, packing, or
	unpacking data through pointers/offsets.  For layered libraries,
	a state update performed by the library may attempt to save the
	selector state, set the selector, update/query some state the
	selector controls, and then restore the selector to its saved state.
	Layered libraries can skip the selector save/restore but this risks
	introducing uncertainty about the state of a selector after calling
	layered library routines.  Such selector side-effects are difficult
	to document and lead to compatibility issues as the layered library
	evolves or its usage varies.  For latched state, layered libraries
	may find commands such as glDrawPixels do not work as expected
	because latched pixel store state is not what the library expects.
	Querying or pushing the latched state, setting the latched state
	explicitly, performing the operation involving latched state, and
	then restoring or popping the latched state avoids entanglements
	with latched state but at considerable cost.
	
	EXAMPLE USAGE OF THIS EXTENSION'S FUNCTIONALITY
	
	Consider the following routine to set the modelview matrix involving
	the matrix mode selector:
	
	  void setModelviewMatrix(const GLfloat matrix[16])
	  {
	    GLenum savedMatrixMode;
	
	    glGetIntegerv(GL_MATRIX_MODE, &savedMatrixMode);
	    glMatrixMode(GL_MODELVIEW);
	    glLoadMatrixf(matrix);
	    glMatrixMode(savedMatrixMode);
	  }
	
	Notice that four OpenGL commands are required to update the current
	modelview matrix without disturbing the matrix mode selector.
	
	OpenGL query commands can also substantially reduce the performance
	of modern OpenGL implementations which may off-load OpenGL state
	processing to another CPU core/thread or to the GPU itself.
	
	An alternative to querying the selector is to use the
	glPushAttrib/glPopAttrib commands.  However this approach typically
	involves pushing far more state than simply the one or two selectors
	that need to be saved and restored.  Because so much state is
	associated with a given push/pop attribute bit, the glPushAttrib
	and glPopAttrib commands are considerably more costly than the
	save/restore approach.  Additionally glPushAttrib risks overflowing
	the attribute stack.
	
	The reliability and performance of layered libraries and applications
	can be improved by adding to the OpenGL API a new set of commands
	to access directly OpenGL state that otherwise involves selectors
	to access.
	
	The above example can be reimplemented more efficiently and without
	selector side-effects:
	
	  void setModelviewMatrix(const GLfloat matrix[16])
	  {
	    glMatrixLoadfEXT(GL_MODELVIEW, matrix);
	  }
	
	Consider a layered library seeking to load a texture:
	
	  void loadTexture(GLint texobj, GLint width, GLint height,
	                   void *data)
	  {
	    glBindTexture(GL_TEXTURE_2D, texobj);
	    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB8,
	                 width, height, GL_RGB, GL_FLOAT, data);
	  }
	
	The library expects the data to be packed into the buffer pointed
	to by data.  But what if the current pixel unpack buffer binding
	is not zero so the current pixel unpack buffer, rather than client
	memory, will be read?  Or what if the application has modified
	the GL_UNPACK_ROW_LENGTH pixel store state before loadTexture
	is called?  
	
	We can fix the routine by calling glBindBuffer(GL_PIXEL_UNPACK_BUFFER,
	0) and setting all the pixel store unpack state to the initial state
	the loadTexture routine expects, but this is expensive.  It also risks
	disturbing the state so when loadTexture returns to the application,
	the application doesn't realize the current texture object (for
	whatever texture unit the current active texture happens to be) and
	pixel store state has changed.
	
	We can more efficiently implement this routine without disturbing
	selector or latched state as follows:
	
	  void loadTexture(GLint texobj, GLint width, GLint height,
	                   void *data)
	  {
	    glPushClientAttribDefaultEXT(GL_CLIENT_PIXEL_STORE_BIT);
	    glTextureImage2D(texobj, GL_TEXTURE_2D, 0, GL_RGB8,
	                     width, height, GL_RGB, GL_FLOAT, data);
	    glPopClientAttrib();
	  }
	
	Now loadTexture does not have to worry about inappropriately
	configured pixel store state or a non-zero pixel unpack buffer
	binding.  And loadTexture has no unintended side-effects for
	selector or latched state (assuming the client attrib state does
	not overflow).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/direct_state_access.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.direct_state_access import *
from OpenGL.raw.GL.EXT.direct_state_access import _EXTENSION_NAME

def glInitDirectStateAccessEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glMatrixLoadfEXT=wrapper.wrapper(glMatrixLoadfEXT).setInputArraySize(
    'm', 16
)
glMatrixLoaddEXT=wrapper.wrapper(glMatrixLoaddEXT).setInputArraySize(
    'm', 16
)
glMatrixMultfEXT=wrapper.wrapper(glMatrixMultfEXT).setInputArraySize(
    'm', 16
)
glMatrixMultdEXT=wrapper.wrapper(glMatrixMultdEXT).setInputArraySize(
    'm', 16
)
# INPUT glTextureParameterfvEXT.params size not checked against 'pname'
glTextureParameterfvEXT=wrapper.wrapper(glTextureParameterfvEXT).setInputArraySize(
    'params', None
)
# INPUT glTextureParameterivEXT.params size not checked against 'pname'
glTextureParameterivEXT=wrapper.wrapper(glTextureParameterivEXT).setInputArraySize(
    'params', None
)
# INPUT glTextureImage1DEXT.pixels size not checked against 'format,type,width'
glTextureImage1DEXT=wrapper.wrapper(glTextureImage1DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glTextureImage2DEXT.pixels size not checked against 'format,type,width,height'
glTextureImage2DEXT=wrapper.wrapper(glTextureImage2DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glTextureSubImage1DEXT.pixels size not checked against 'format,type,width'
glTextureSubImage1DEXT=wrapper.wrapper(glTextureSubImage1DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glTextureSubImage2DEXT.pixels size not checked against 'format,type,width,height'
glTextureSubImage2DEXT=wrapper.wrapper(glTextureSubImage2DEXT).setInputArraySize(
    'pixels', None
)
# OUTPUT glGetTextureImageEXT.pixels COMPSIZE(target, level, format, type) 
glGetTextureParameterfvEXT=wrapper.wrapper(glGetTextureParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTextureParameterivEXT=wrapper.wrapper(glGetTextureParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTextureLevelParameterfvEXT=wrapper.wrapper(glGetTextureLevelParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTextureLevelParameterivEXT=wrapper.wrapper(glGetTextureLevelParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glTextureImage3DEXT.pixels size not checked against 'format,type,width,height,depth'
glTextureImage3DEXT=wrapper.wrapper(glTextureImage3DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glTextureSubImage3DEXT.pixels size not checked against 'format,type,width,height,depth'
glTextureSubImage3DEXT=wrapper.wrapper(glTextureSubImage3DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glMultiTexCoordPointerEXT.pointer size not checked against 'size,type,stride'
glMultiTexCoordPointerEXT=wrapper.wrapper(glMultiTexCoordPointerEXT).setInputArraySize(
    'pointer', None
)
# INPUT glMultiTexEnvfvEXT.params size not checked against 'pname'
glMultiTexEnvfvEXT=wrapper.wrapper(glMultiTexEnvfvEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexEnvivEXT.params size not checked against 'pname'
glMultiTexEnvivEXT=wrapper.wrapper(glMultiTexEnvivEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexGendvEXT.params size not checked against 'pname'
glMultiTexGendvEXT=wrapper.wrapper(glMultiTexGendvEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexGenfvEXT.params size not checked against 'pname'
glMultiTexGenfvEXT=wrapper.wrapper(glMultiTexGenfvEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexGenivEXT.params size not checked against 'pname'
glMultiTexGenivEXT=wrapper.wrapper(glMultiTexGenivEXT).setInputArraySize(
    'params', None
)
glGetMultiTexEnvfvEXT=wrapper.wrapper(glGetMultiTexEnvfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexEnvivEXT=wrapper.wrapper(glGetMultiTexEnvivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexGendvEXT=wrapper.wrapper(glGetMultiTexGendvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexGenfvEXT=wrapper.wrapper(glGetMultiTexGenfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexGenivEXT=wrapper.wrapper(glGetMultiTexGenivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glMultiTexParameterivEXT.params size not checked against 'pname'
glMultiTexParameterivEXT=wrapper.wrapper(glMultiTexParameterivEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexParameterfvEXT.params size not checked against 'pname'
glMultiTexParameterfvEXT=wrapper.wrapper(glMultiTexParameterfvEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexImage1DEXT.pixels size not checked against 'format,type,width'
glMultiTexImage1DEXT=wrapper.wrapper(glMultiTexImage1DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glMultiTexImage2DEXT.pixels size not checked against 'format,type,width,height'
glMultiTexImage2DEXT=wrapper.wrapper(glMultiTexImage2DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glMultiTexSubImage1DEXT.pixels size not checked against 'format,type,width'
glMultiTexSubImage1DEXT=wrapper.wrapper(glMultiTexSubImage1DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glMultiTexSubImage2DEXT.pixels size not checked against 'format,type,width,height'
glMultiTexSubImage2DEXT=wrapper.wrapper(glMultiTexSubImage2DEXT).setInputArraySize(
    'pixels', None
)
# OUTPUT glGetMultiTexImageEXT.pixels COMPSIZE(target, level, format, type) 
glGetMultiTexParameterfvEXT=wrapper.wrapper(glGetMultiTexParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexParameterivEXT=wrapper.wrapper(glGetMultiTexParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexLevelParameterfvEXT=wrapper.wrapper(glGetMultiTexLevelParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexLevelParameterivEXT=wrapper.wrapper(glGetMultiTexLevelParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glMultiTexImage3DEXT.pixels size not checked against 'format,type,width,height,depth'
glMultiTexImage3DEXT=wrapper.wrapper(glMultiTexImage3DEXT).setInputArraySize(
    'pixels', None
)
# INPUT glMultiTexSubImage3DEXT.pixels size not checked against 'format,type,width,height,depth'
glMultiTexSubImage3DEXT=wrapper.wrapper(glMultiTexSubImage3DEXT).setInputArraySize(
    'pixels', None
)
glGetFloatIndexedvEXT=wrapper.wrapper(glGetFloatIndexedvEXT).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='target',orPassIn=True
)
glGetDoubleIndexedvEXT=wrapper.wrapper(glGetDoubleIndexedvEXT).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='target',orPassIn=True
)
glGetPointerIndexedvEXT=wrapper.wrapper(glGetPointerIndexedvEXT).setOutput(
    'data',size=(1,),orPassIn=True
)
glGetIntegerIndexedvEXT=wrapper.wrapper(glGetIntegerIndexedvEXT).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='target',orPassIn=True
)
glGetBooleanIndexedvEXT=wrapper.wrapper(glGetBooleanIndexedvEXT).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='target',orPassIn=True
)
# INPUT glCompressedTextureImage3DEXT.bits size not checked against imageSize
glCompressedTextureImage3DEXT=wrapper.wrapper(glCompressedTextureImage3DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedTextureImage2DEXT.bits size not checked against imageSize
glCompressedTextureImage2DEXT=wrapper.wrapper(glCompressedTextureImage2DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedTextureImage1DEXT.bits size not checked against imageSize
glCompressedTextureImage1DEXT=wrapper.wrapper(glCompressedTextureImage1DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedTextureSubImage3DEXT.bits size not checked against imageSize
glCompressedTextureSubImage3DEXT=wrapper.wrapper(glCompressedTextureSubImage3DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedTextureSubImage2DEXT.bits size not checked against imageSize
glCompressedTextureSubImage2DEXT=wrapper.wrapper(glCompressedTextureSubImage2DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedTextureSubImage1DEXT.bits size not checked against imageSize
glCompressedTextureSubImage1DEXT=wrapper.wrapper(glCompressedTextureSubImage1DEXT).setInputArraySize(
    'bits', None
)
# OUTPUT glGetCompressedTextureImageEXT.img COMPSIZE(target, lod) 
# INPUT glCompressedMultiTexImage3DEXT.bits size not checked against imageSize
glCompressedMultiTexImage3DEXT=wrapper.wrapper(glCompressedMultiTexImage3DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedMultiTexImage2DEXT.bits size not checked against imageSize
glCompressedMultiTexImage2DEXT=wrapper.wrapper(glCompressedMultiTexImage2DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedMultiTexImage1DEXT.bits size not checked against imageSize
glCompressedMultiTexImage1DEXT=wrapper.wrapper(glCompressedMultiTexImage1DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedMultiTexSubImage3DEXT.bits size not checked against imageSize
glCompressedMultiTexSubImage3DEXT=wrapper.wrapper(glCompressedMultiTexSubImage3DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedMultiTexSubImage2DEXT.bits size not checked against imageSize
glCompressedMultiTexSubImage2DEXT=wrapper.wrapper(glCompressedMultiTexSubImage2DEXT).setInputArraySize(
    'bits', None
)
# INPUT glCompressedMultiTexSubImage1DEXT.bits size not checked against imageSize
glCompressedMultiTexSubImage1DEXT=wrapper.wrapper(glCompressedMultiTexSubImage1DEXT).setInputArraySize(
    'bits', None
)
# OUTPUT glGetCompressedMultiTexImageEXT.img COMPSIZE(target, lod) 
glMatrixLoadTransposefEXT=wrapper.wrapper(glMatrixLoadTransposefEXT).setInputArraySize(
    'm', 16
)
glMatrixLoadTransposedEXT=wrapper.wrapper(glMatrixLoadTransposedEXT).setInputArraySize(
    'm', 16
)
glMatrixMultTransposefEXT=wrapper.wrapper(glMatrixMultTransposefEXT).setInputArraySize(
    'm', 16
)
glMatrixMultTransposedEXT=wrapper.wrapper(glMatrixMultTransposedEXT).setInputArraySize(
    'm', 16
)
# INPUT glNamedBufferDataEXT.data size not checked against 'size'
glNamedBufferDataEXT=wrapper.wrapper(glNamedBufferDataEXT).setInputArraySize(
    'data', None
)
# INPUT glNamedBufferSubDataEXT.data size not checked against 'size'
glNamedBufferSubDataEXT=wrapper.wrapper(glNamedBufferSubDataEXT).setInputArraySize(
    'data', None
)
glGetNamedBufferParameterivEXT=wrapper.wrapper(glGetNamedBufferParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetNamedBufferPointervEXT=wrapper.wrapper(glGetNamedBufferPointervEXT).setOutput(
    'params',size=(1,),orPassIn=True
)
glGetNamedBufferSubDataEXT=wrapper.wrapper(glGetNamedBufferSubDataEXT).setOutput(
    'data',size=_glgets._glget_size_mapping,pnameArg='size',orPassIn=True
)
# INPUT glProgramUniform1fvEXT.value size not checked against count
glProgramUniform1fvEXT=wrapper.wrapper(glProgramUniform1fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2fvEXT.value size not checked against count*2
glProgramUniform2fvEXT=wrapper.wrapper(glProgramUniform2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3fvEXT.value size not checked against count*3
glProgramUniform3fvEXT=wrapper.wrapper(glProgramUniform3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4fvEXT.value size not checked against count*4
glProgramUniform4fvEXT=wrapper.wrapper(glProgramUniform4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1ivEXT.value size not checked against count
glProgramUniform1ivEXT=wrapper.wrapper(glProgramUniform1ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2ivEXT.value size not checked against count*2
glProgramUniform2ivEXT=wrapper.wrapper(glProgramUniform2ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3ivEXT.value size not checked against count*3
glProgramUniform3ivEXT=wrapper.wrapper(glProgramUniform3ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4ivEXT.value size not checked against count*4
glProgramUniform4ivEXT=wrapper.wrapper(glProgramUniform4ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2fvEXT.value size not checked against count*4
glProgramUniformMatrix2fvEXT=wrapper.wrapper(glProgramUniformMatrix2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3fvEXT.value size not checked against count*9
glProgramUniformMatrix3fvEXT=wrapper.wrapper(glProgramUniformMatrix3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fvEXT.value size not checked against count*16
glProgramUniformMatrix4fvEXT=wrapper.wrapper(glProgramUniformMatrix4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x3fvEXT.value size not checked against count*6
glProgramUniformMatrix2x3fvEXT=wrapper.wrapper(glProgramUniformMatrix2x3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x2fvEXT.value size not checked against count*6
glProgramUniformMatrix3x2fvEXT=wrapper.wrapper(glProgramUniformMatrix3x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x4fvEXT.value size not checked against count*8
glProgramUniformMatrix2x4fvEXT=wrapper.wrapper(glProgramUniformMatrix2x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x2fvEXT.value size not checked against count*8
glProgramUniformMatrix4x2fvEXT=wrapper.wrapper(glProgramUniformMatrix4x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x4fvEXT.value size not checked against count*12
glProgramUniformMatrix3x4fvEXT=wrapper.wrapper(glProgramUniformMatrix3x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x3fvEXT.value size not checked against count*12
glProgramUniformMatrix4x3fvEXT=wrapper.wrapper(glProgramUniformMatrix4x3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glTextureParameterIivEXT.params size not checked against 'pname'
glTextureParameterIivEXT=wrapper.wrapper(glTextureParameterIivEXT).setInputArraySize(
    'params', None
)
# INPUT glTextureParameterIuivEXT.params size not checked against 'pname'
glTextureParameterIuivEXT=wrapper.wrapper(glTextureParameterIuivEXT).setInputArraySize(
    'params', None
)
glGetTextureParameterIivEXT=wrapper.wrapper(glGetTextureParameterIivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTextureParameterIuivEXT=wrapper.wrapper(glGetTextureParameterIuivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glMultiTexParameterIivEXT.params size not checked against 'pname'
glMultiTexParameterIivEXT=wrapper.wrapper(glMultiTexParameterIivEXT).setInputArraySize(
    'params', None
)
# INPUT glMultiTexParameterIuivEXT.params size not checked against 'pname'
glMultiTexParameterIuivEXT=wrapper.wrapper(glMultiTexParameterIuivEXT).setInputArraySize(
    'params', None
)
glGetMultiTexParameterIivEXT=wrapper.wrapper(glGetMultiTexParameterIivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMultiTexParameterIuivEXT=wrapper.wrapper(glGetMultiTexParameterIuivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glProgramUniform1uivEXT.value size not checked against count
glProgramUniform1uivEXT=wrapper.wrapper(glProgramUniform1uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2uivEXT.value size not checked against count*2
glProgramUniform2uivEXT=wrapper.wrapper(glProgramUniform2uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3uivEXT.value size not checked against count*3
glProgramUniform3uivEXT=wrapper.wrapper(glProgramUniform3uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4uivEXT.value size not checked against count*4
glProgramUniform4uivEXT=wrapper.wrapper(glProgramUniform4uivEXT).setInputArraySize(
    'value', None
)
# INPUT glNamedProgramLocalParameters4fvEXT.params size not checked against count*4
glNamedProgramLocalParameters4fvEXT=wrapper.wrapper(glNamedProgramLocalParameters4fvEXT).setInputArraySize(
    'params', None
)
glNamedProgramLocalParameterI4ivEXT=wrapper.wrapper(glNamedProgramLocalParameterI4ivEXT).setInputArraySize(
    'params', 4
)
# INPUT glNamedProgramLocalParametersI4ivEXT.params size not checked against count*4
glNamedProgramLocalParametersI4ivEXT=wrapper.wrapper(glNamedProgramLocalParametersI4ivEXT).setInputArraySize(
    'params', None
)
glNamedProgramLocalParameterI4uivEXT=wrapper.wrapper(glNamedProgramLocalParameterI4uivEXT).setInputArraySize(
    'params', 4
)
# INPUT glNamedProgramLocalParametersI4uivEXT.params size not checked against count*4
glNamedProgramLocalParametersI4uivEXT=wrapper.wrapper(glNamedProgramLocalParametersI4uivEXT).setInputArraySize(
    'params', None
)
glGetNamedProgramLocalParameterIivEXT=wrapper.wrapper(glGetNamedProgramLocalParameterIivEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetNamedProgramLocalParameterIuivEXT=wrapper.wrapper(glGetNamedProgramLocalParameterIuivEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
# INPUT glGetFloati_vEXT.params size not checked against 'pname'
glGetFloati_vEXT=wrapper.wrapper(glGetFloati_vEXT).setInputArraySize(
    'params', None
)
# INPUT glGetDoublei_vEXT.params size not checked against 'pname'
glGetDoublei_vEXT=wrapper.wrapper(glGetDoublei_vEXT).setInputArraySize(
    'params', None
)
glGetPointeri_vEXT=wrapper.wrapper(glGetPointeri_vEXT).setInputArraySize(
    'params', 1
)
# INPUT glNamedProgramStringEXT.string size not checked against len
glNamedProgramStringEXT=wrapper.wrapper(glNamedProgramStringEXT).setInputArraySize(
    'string', None
)
glNamedProgramLocalParameter4dvEXT=wrapper.wrapper(glNamedProgramLocalParameter4dvEXT).setInputArraySize(
    'params', 4
)
glNamedProgramLocalParameter4fvEXT=wrapper.wrapper(glNamedProgramLocalParameter4fvEXT).setInputArraySize(
    'params', 4
)
glGetNamedProgramLocalParameterdvEXT=wrapper.wrapper(glGetNamedProgramLocalParameterdvEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetNamedProgramLocalParameterfvEXT=wrapper.wrapper(glGetNamedProgramLocalParameterfvEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
glGetNamedProgramivEXT=wrapper.wrapper(glGetNamedProgramivEXT).setOutput(
    'params',size=(1,),orPassIn=True
)
# OUTPUT glGetNamedProgramStringEXT.string COMPSIZE(program, pname) 
glGetNamedRenderbufferParameterivEXT=wrapper.wrapper(glGetNamedRenderbufferParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetNamedFramebufferAttachmentParameterivEXT=wrapper.wrapper(glGetNamedFramebufferAttachmentParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glFramebufferDrawBuffersEXT.bufs size not checked against n
glFramebufferDrawBuffersEXT=wrapper.wrapper(glFramebufferDrawBuffersEXT).setInputArraySize(
    'bufs', None
)
glGetFramebufferParameterivEXT=wrapper.wrapper(glGetFramebufferParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetVertexArrayPointervEXT=wrapper.wrapper(glGetVertexArrayPointervEXT).setInputArraySize(
    'param', 1
)
# INPUT glNamedBufferStorageEXT.data size not checked against size
glNamedBufferStorageEXT=wrapper.wrapper(glNamedBufferStorageEXT).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferDataEXT.data size not checked against 'format,type'
glClearNamedBufferDataEXT=wrapper.wrapper(glClearNamedBufferDataEXT).setInputArraySize(
    'data', None
)
# INPUT glClearNamedBufferSubDataEXT.data size not checked against 'format,type'
glClearNamedBufferSubDataEXT=wrapper.wrapper(glClearNamedBufferSubDataEXT).setInputArraySize(
    'data', None
)
glGetNamedFramebufferParameterivEXT=wrapper.wrapper(glGetNamedFramebufferParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# INPUT glProgramUniform1dvEXT.value size not checked against count
glProgramUniform1dvEXT=wrapper.wrapper(glProgramUniform1dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2dvEXT.value size not checked against count
glProgramUniform2dvEXT=wrapper.wrapper(glProgramUniform2dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3dvEXT.value size not checked against count
glProgramUniform3dvEXT=wrapper.wrapper(glProgramUniform3dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4dvEXT.value size not checked against count
glProgramUniform4dvEXT=wrapper.wrapper(glProgramUniform4dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2dvEXT.value size not checked against count
glProgramUniformMatrix2dvEXT=wrapper.wrapper(glProgramUniformMatrix2dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3dvEXT.value size not checked against count
glProgramUniformMatrix3dvEXT=wrapper.wrapper(glProgramUniformMatrix3dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4dvEXT.value size not checked against count
glProgramUniformMatrix4dvEXT=wrapper.wrapper(glProgramUniformMatrix4dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x3dvEXT.value size not checked against count
glProgramUniformMatrix2x3dvEXT=wrapper.wrapper(glProgramUniformMatrix2x3dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x4dvEXT.value size not checked against count
glProgramUniformMatrix2x4dvEXT=wrapper.wrapper(glProgramUniformMatrix2x4dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x2dvEXT.value size not checked against count
glProgramUniformMatrix3x2dvEXT=wrapper.wrapper(glProgramUniformMatrix3x2dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x4dvEXT.value size not checked against count
glProgramUniformMatrix3x4dvEXT=wrapper.wrapper(glProgramUniformMatrix3x4dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x2dvEXT.value size not checked against count
glProgramUniformMatrix4x2dvEXT=wrapper.wrapper(glProgramUniformMatrix4x2dvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x3dvEXT.value size not checked against count
glProgramUniformMatrix4x3dvEXT=wrapper.wrapper(glProgramUniformMatrix4x3dvEXT).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.query_buffer_object

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.query_buffer_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	Statistics about the operation of the OpenGL pipeline, such as the number
	of samples that passed the depth test, the elapsed time between two events
	or the number of vertices written by transform feedback can be retrieved
	from the GL through query objects. The result of a query object is
	acquired by the application through the OpenGL API into a client provided
	memory location. Should the result returned by the API be required for use
	in a shader, it must be passed back to the GL via a program uniform or
	some other mechanism. This requires a round-trip from the GPU to the CPU
	and back.
	
	This extension introduces a mechanism whereby the result of a query object
	may be retrieved into a buffer object instead of client memory. This allows
	the query rsult to be made available to a shader without a round-trip to
	the CPU for example by subsequently using the buffer object as a uniform
	buffer, texture buffer or other data store visible to the shader. This
	functionality may also be used to place the results of many query objects
	into a single, large buffer and then map or otherwise read back the entire
	buffer at a later point in time, avoiding a per-query object CPU-GPU
	synchronization event.
	
	The extension allows acquiring the result of any query object type
	supported by the GL implementation into a buffer object. The implementation
	will determine the most efficient method of copying the query result to the
	buffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/query_buffer_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.query_buffer_object import *
from OpenGL.raw.GL.ARB.query_buffer_object import _EXTENSION_NAME

def glInitQueryBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_framebuffer_object'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_framebuffer_object',error_checker=_errors._error_checker)
GL_COLOR_ATTACHMENT0_OES=_C('GL_COLOR_ATTACHMENT0_OES',0x8CE0)
GL_DEPTH_ATTACHMENT_OES=_C('GL_DEPTH_ATTACHMENT_OES',0x8D00)
GL_DEPTH_COMPONENT16_OES=_C('GL_DEPTH_COMPONENT16_OES',0x81A5)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_OES=_C('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_OES',0x8CD1)
GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_OES=_C('GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_OES',0x8CD0)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_OES=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE_OES',0x8CD3)
GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_OES=_C('GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL_OES',0x8CD2)
GL_FRAMEBUFFER_BINDING_OES=_C('GL_FRAMEBUFFER_BINDING_OES',0x8CA6)
GL_FRAMEBUFFER_COMPLETE_OES=_C('GL_FRAMEBUFFER_COMPLETE_OES',0x8CD5)
GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_OES=_C('GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_OES',0x8CD6)
GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_OES=_C('GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS_OES',0x8CD9)
GL_FRAMEBUFFER_INCOMPLETE_FORMATS_OES=_C('GL_FRAMEBUFFER_INCOMPLETE_FORMATS_OES',0x8CDA)
GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_OES=_C('GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_OES',0x8CD7)
GL_FRAMEBUFFER_OES=_C('GL_FRAMEBUFFER_OES',0x8D40)
GL_FRAMEBUFFER_UNSUPPORTED_OES=_C('GL_FRAMEBUFFER_UNSUPPORTED_OES',0x8CDD)
GL_INVALID_FRAMEBUFFER_OPERATION_OES=_C('GL_INVALID_FRAMEBUFFER_OPERATION_OES',0x0506)
GL_MAX_RENDERBUFFER_SIZE_OES=_C('GL_MAX_RENDERBUFFER_SIZE_OES',0x84E8)
GL_NONE_OES=_C('GL_NONE_OES',0)
GL_RENDERBUFFER_ALPHA_SIZE_OES=_C('GL_RENDERBUFFER_ALPHA_SIZE_OES',0x8D53)
GL_RENDERBUFFER_BINDING_OES=_C('GL_RENDERBUFFER_BINDING_OES',0x8CA7)
GL_RENDERBUFFER_BLUE_SIZE_OES=_C('GL_RENDERBUFFER_BLUE_SIZE_OES',0x8D52)
GL_RENDERBUFFER_DEPTH_SIZE_OES=_C('GL_RENDERBUFFER_DEPTH_SIZE_OES',0x8D54)
GL_RENDERBUFFER_GREEN_SIZE_OES=_C('GL_RENDERBUFFER_GREEN_SIZE_OES',0x8D51)
GL_RENDERBUFFER_HEIGHT_OES=_C('GL_RENDERBUFFER_HEIGHT_OES',0x8D43)
GL_RENDERBUFFER_INTERNAL_FORMAT_OES=_C('GL_RENDERBUFFER_INTERNAL_FORMAT_OES',0x8D44)
GL_RENDERBUFFER_OES=_C('GL_RENDERBUFFER_OES',0x8D41)
GL_RENDERBUFFER_RED_SIZE_OES=_C('GL_RENDERBUFFER_RED_SIZE_OES',0x8D50)
GL_RENDERBUFFER_STENCIL_SIZE_OES=_C('GL_RENDERBUFFER_STENCIL_SIZE_OES',0x8D55)
GL_RENDERBUFFER_WIDTH_OES=_C('GL_RENDERBUFFER_WIDTH_OES',0x8D42)
GL_RGB565_OES=_C('GL_RGB565_OES',0x8D62)
GL_RGB5_A1_OES=_C('GL_RGB5_A1_OES',0x8057)
GL_RGBA4_OES=_C('GL_RGBA4_OES',0x8056)
GL_STENCIL_ATTACHMENT_OES=_C('GL_STENCIL_ATTACHMENT_OES',0x8D20)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindFramebufferOES(target,framebuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindRenderbufferOES(target,renderbuffer):pass
@_f
@_p.types(_cs.GLenum,_cs.GLenum)
def glCheckFramebufferStatusOES(target):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteFramebuffersOES(n,framebuffers):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteRenderbuffersOES(n,renderbuffers):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint)
def glFramebufferRenderbufferOES(target,attachment,renderbuffertarget,renderbuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLint)
def glFramebufferTexture2DOES(target,attachment,textarget,texture,level):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenFramebuffersOES(n,framebuffers):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenRenderbuffersOES(n,renderbuffers):pass
@_f
@_p.types(None,_cs.GLenum)
def glGenerateMipmapOES(target):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetFramebufferAttachmentParameterivOES(target,attachment,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetRenderbufferParameterivOES(target,pname,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsFramebufferOES(framebuffer):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsRenderbufferOES(renderbuffer):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLsizei,_cs.GLsizei)
def glRenderbufferStorageOES(target,internalformat,width,height):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_NV_blend_equation_advanced'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_NV_blend_equation_advanced',error_checker=_errors._error_checker)
GL_BLEND_OVERLAP_NV=_C('GL_BLEND_OVERLAP_NV',0x9281)
GL_BLEND_PREMULTIPLIED_SRC_NV=_C('GL_BLEND_PREMULTIPLIED_SRC_NV',0x9280)
GL_BLUE_NV=_C('GL_BLUE_NV',0x1905)
GL_COLORBURN_NV=_C('GL_COLORBURN_NV',0x929A)
GL_COLORDODGE_NV=_C('GL_COLORDODGE_NV',0x9299)
GL_CONJOINT_NV=_C('GL_CONJOINT_NV',0x9284)
GL_CONTRAST_NV=_C('GL_CONTRAST_NV',0x92A1)
GL_DARKEN_NV=_C('GL_DARKEN_NV',0x9297)
GL_DIFFERENCE_NV=_C('GL_DIFFERENCE_NV',0x929E)
GL_DISJOINT_NV=_C('GL_DISJOINT_NV',0x9283)
GL_DST_ATOP_NV=_C('GL_DST_ATOP_NV',0x928F)
GL_DST_IN_NV=_C('GL_DST_IN_NV',0x928B)
GL_DST_NV=_C('GL_DST_NV',0x9287)
GL_DST_OUT_NV=_C('GL_DST_OUT_NV',0x928D)
GL_DST_OVER_NV=_C('GL_DST_OVER_NV',0x9289)
GL_EXCLUSION_NV=_C('GL_EXCLUSION_NV',0x92A0)
GL_GREEN_NV=_C('GL_GREEN_NV',0x1904)
GL_HARDLIGHT_NV=_C('GL_HARDLIGHT_NV',0x929B)
GL_HARDMIX_NV=_C('GL_HARDMIX_NV',0x92A9)
GL_HSL_COLOR_NV=_C('GL_HSL_COLOR_NV',0x92AF)
GL_HSL_HUE_NV=_C('GL_HSL_HUE_NV',0x92AD)
GL_HSL_LUMINOSITY_NV=_C('GL_HSL_LUMINOSITY_NV',0x92B0)
GL_HSL_SATURATION_NV=_C('GL_HSL_SATURATION_NV',0x92AE)
GL_INVERT=_C('GL_INVERT',0x150A)
GL_INVERT_OVG_NV=_C('GL_INVERT_OVG_NV',0x92B4)
GL_INVERT_RGB_NV=_C('GL_INVERT_RGB_NV',0x92A3)
GL_LIGHTEN_NV=_C('GL_LIGHTEN_NV',0x9298)
GL_LINEARBURN_NV=_C('GL_LINEARBURN_NV',0x92A5)
GL_LINEARDODGE_NV=_C('GL_LINEARDODGE_NV',0x92A4)
GL_LINEARLIGHT_NV=_C('GL_LINEARLIGHT_NV',0x92A7)
GL_MINUS_CLAMPED_NV=_C('GL_MINUS_CLAMPED_NV',0x92B3)
GL_MINUS_NV=_C('GL_MINUS_NV',0x929F)
GL_MULTIPLY_NV=_C('GL_MULTIPLY_NV',0x9294)
GL_OVERLAY_NV=_C('GL_OVERLAY_NV',0x9296)
GL_PINLIGHT_NV=_C('GL_PINLIGHT_NV',0x92A8)
GL_PLUS_CLAMPED_ALPHA_NV=_C('GL_PLUS_CLAMPED_ALPHA_NV',0x92B2)
GL_PLUS_CLAMPED_NV=_C('GL_PLUS_CLAMPED_NV',0x92B1)
GL_PLUS_DARKER_NV=_C('GL_PLUS_DARKER_NV',0x9292)
GL_PLUS_NV=_C('GL_PLUS_NV',0x9291)
GL_RED_NV=_C('GL_RED_NV',0x1903)
GL_SCREEN_NV=_C('GL_SCREEN_NV',0x9295)
GL_SOFTLIGHT_NV=_C('GL_SOFTLIGHT_NV',0x929C)
GL_SRC_ATOP_NV=_C('GL_SRC_ATOP_NV',0x928E)
GL_SRC_IN_NV=_C('GL_SRC_IN_NV',0x928A)
GL_SRC_NV=_C('GL_SRC_NV',0x9286)
GL_SRC_OUT_NV=_C('GL_SRC_OUT_NV',0x928C)
GL_SRC_OVER_NV=_C('GL_SRC_OVER_NV',0x9288)
GL_UNCORRELATED_NV=_C('GL_UNCORRELATED_NV',0x9282)
GL_VIVIDLIGHT_NV=_C('GL_VIVIDLIGHT_NV',0x92A6)
GL_XOR_NV=_C('GL_XOR_NV',0x1506)
GL_ZERO=_C('GL_ZERO',0)
@_f
@_p.types(None,)
def glBlendBarrierNV():pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint)
def glBlendParameteriNV(pname,value):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_shader_buffer_load'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_shader_buffer_load',error_checker=_errors._error_checker)
GL_BUFFER_GPU_ADDRESS_NV=_C('GL_BUFFER_GPU_ADDRESS_NV',0x8F1D)
GL_GPU_ADDRESS_NV=_C('GL_GPU_ADDRESS_NV',0x8F34)
GL_MAX_SHADER_BUFFER_ADDRESS_NV=_C('GL_MAX_SHADER_BUFFER_ADDRESS_NV',0x8F35)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLuint64Array)
def glGetBufferParameterui64vNV(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLuint64Array)
def glGetIntegerui64vNV(value,result):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,arrays.GLuint64Array)
def glGetNamedBufferParameterui64vNV(buffer,pname,params):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,arrays.GLuint64Array)
def glGetUniformui64vNV(program,location,params):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLenum)
def glIsBufferResidentNV(target):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsNamedBufferResidentNV(buffer):pass
@_f
@_p.types(None,_cs.GLenum)
def glMakeBufferNonResidentNV(target):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum)
def glMakeBufferResidentNV(target,access):pass
@_f
@_p.types(None,_cs.GLuint)
def glMakeNamedBufferNonResidentNV(buffer):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum)
def glMakeNamedBufferResidentNV(buffer,access):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64EXT)
def glProgramUniformui64NV(program,location,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniformui64vNV(program,location,count,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64EXT)
def glUniformui64NV(location,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniformui64vNV(location,count,value):pass
